import sys
import webbrowser
import time
from exc_analyzer.print_utils import print_info, print_success, print_warning, print_error, print_action, print_cancelled, safe_print, colorize, clear_screen, _write_output
from exc_analyzer.i18n import t, get_active_language
from exc_analyzer.api import exchange_device_code, poll_for_token, fetch_github_user
from exc_analyzer.config import save_key
from exc_analyzer.spinner import Spinner
def cmd_login(args):
    """
    Handle the GitHub Device Flow login.
    """
    from exc_analyzer.config import load_key
    existing_key = load_key()
    if existing_key:
        user = fetch_github_user(existing_key)
        if user:
            safe_print("")
            _write_output("")
            print_success(t("commands.login.already_logged_in", user=colorize(user, '1')))
            safe_print("")
            _write_output("")
            safe_print(t("commands.login.already_login_hint"))
            _write_output(t("commands.login.already_login_hint"))
            safe_print("")
            _write_output("")
            sys.exit(0)
    safe_print("")
    _write_output("")
    disclaimer = t("commands.login.security_disclaimer")
    safe_print(colorize(disclaimer, '90'))
    _write_output(disclaimer)
    safe_print("")
    _write_output("")
    agree_text = colorize(t("commands.login.agree_prompt"), '97')
    try:
        if sys.version_info.major < 3:
            response = raw_input(agree_text)
        else:
            response = input(agree_text)
    except KeyboardInterrupt:
        clear_screen()
        print_cancelled(t("commands.login.cancelled"))
        safe_print("")
        sys.exit(0)
    lang = get_active_language()
    valid_responses = ['y', 'yes']
    if lang == 'tr':
        valid_responses = ['e', 'evet']
    if response.lower() not in valid_responses:
        clear_screen()
        print_cancelled(t("commands.login.cancelled"))
        safe_print("")
        sys.exit(0)
    device_data = exchange_device_code()
    if not device_data:
        sys.exit(1)
    user_code = device_data['user_code']
    verification_uri = device_data['verification_uri']
    interval = device_data['interval']
    device_code = device_data['device_code']
    safe_print("\n" + "="*40)
    _write_output("\n" + "="*40)
    print_action(t("commands.login.otp_code", code=colorize(user_code, '93')))
    safe_print("="*40 + "\n")
    _write_output("="*40 + "\n")
    try:
        import pyperclip
        pyperclip.copy(user_code)
        print_success(t("commands.login.clipboard_copy"))
    except ImportError:
        pass
    except Exception:
        pass
    safe_print(f"URL: {colorize(verification_uri, '96')}\n")
    _write_output(f"URL: {verification_uri}\n")
    try:
        sys.stdout.flush()
        input(t("commands.login.open_browser"))
    except KeyboardInterrupt:
        clear_screen()
        print_cancelled(t("commands.login.cancelled"))
        safe_print("")
        sys.exit(0)
    webbrowser.open(verification_uri)
    token = None
    try:
        with Spinner(t("commands.login.waiting")):
            token = poll_for_token(device_code, interval)
    except KeyboardInterrupt:
        clear_screen()
        print_cancelled(t("commands.login.cancelled"))
        safe_print("")
        sys.exit(0)
    sys.stdout.flush()
    if token:
        clear_screen()
        save_key(token, silent=True)
        user = fetch_github_user(token) or "Unknown"
        print_success(t("commands.login.success", user=colorize(user, '1')))
        hint = t("commands.login.suggestion")
        safe_print(colorize(hint, '93'))
        _write_output(hint)
        safe_print("")
        _write_output("")
    else:
        clear_screen()
        print_cancelled(t("commands.login.cancelled"))
        safe_print("")
        sys.exit(1)



"""GitHub GraphQL API client for batch queries and high-performance analysis."""
from typing import Optional, Dict, Any
class GraphQLClient:
    """GitHub GraphQL API batch query builder for high-performance analysis."""
    REPO_ANALYSIS_QUERY = """
    query RepoAnalysis($owner: String!, $name: String!) {
      repository(owner: $owner, name: $name) {
        description
        createdAt
        updatedAt
        stargazerCount
        forkCount
        defaultBranchRef {
          name
        }
        licenseInfo {
          name
        }
        issues {
          totalCount
        }
        pullRequests {
          totalCount
        }
        primaryLanguage {
          name
        }
        languages(first: 20) {
          edges {
            node {
              name
            }
            size
          }
        }
      }
    }
    """
    COMMITS_QUERY = """
    query CommitHistory($owner: String!, $name: String!, $ref: String!, $first: Int!) {
      repository(owner: $owner, name: $name) {
        ref(qualifiedName: $ref) {
          target {
            ... on Commit {
              history(first: $first) {
                pageInfo {
                  endCursor
                  hasNextPage
                }
                edges {
                  node {
                    oid
                    message
                    author {
                      name
                      user {
                        login
                      }
                    }
                    committedDate
                    additions
                    deletions
                  }
                }
              }
            }
          }
        }
      }
    }
    """
    CONTRIBUTORS_QUERY = """
    query Contributors($owner: String!, $name: String!, $first: Int!) {
      repository(owner: $owner, name: $name) {
        defaultBranchRef {
          target {
            ... on Commit {
              history(first: $first) {
                edges {
                  node {
                    author {
                      user {
                        login
                        name
                      }
                      date
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    """
    @staticmethod
    def build_repo_query(owner: str, name: str) -> Dict[str, Any]:
        """Build GraphQL query for repository analysis."""
        return {
            "query": GraphQLClient.REPO_ANALYSIS_QUERY,
            "variables": {
                "owner": owner,
                "name": name
            }
        }
    @staticmethod
    def build_commits_query(owner: str, name: str, ref: str, first: int = 100, after: Optional[str] = None) -> Dict[str, Any]:
        """Build GraphQL query for commit history with pagination."""
        query = """
        query CommitHistory($owner: String!, $name: String!, $ref: String!, $first: Int!, $after: String) {
          repository(owner: $owner, name: $name) {
            ref(qualifiedName: $ref) {
              target {
                ... on Commit {
                  history(first: $first, after: $after) {
                    pageInfo {
                      endCursor
                      hasNextPage
                    }
                    edges {
                      node {
                        oid
                        message
                        author {
                          name
                          user {
                            login
                          }
                        }
                        committedDate
                        additions
                        deletions
                      }
                    }
                  }
                }
              }
            }
          }
        }
        """
        return {
            "query": query,
            "variables": {
                "owner": owner,
                "name": name,
                "ref": ref,
                "first": first,
                "after": after
            }
        }
    @staticmethod
    def build_contributors_query(owner: str, name: str, first: int = 100) -> Dict[str, Any]:
        """Build GraphQL query for contributors."""
        return {
            "query": GraphQLClient.CONTRIBUTORS_QUERY,
            "variables": {
                "owner": owner,
                "name": name,
                "first": first
            }
        }
    @staticmethod
    def build_search_commits_query(query_string: str) -> Dict[str, Any]:
        """Build GraphQL query for commit search."""
        return {
            "query": """
            query SearchCommits($query: String!, $first: Int!) {
              search(query: $query, type: ISSUE, first: $first) {
                issueCount
              }
            }
            """,
            "variables": {
                "query": query_string,
                "first": 100
            }
        }



def mask_sensitive(text: str) -> str:
    """Mask known sensitive patterns before logging/displaying."""
    try:
        import re
        text = re.sub(r"ghp_[A-Za-z0-9]{36}", "ghp_********************", text)
        text = re.sub(r"(?i)(aws_secret_access_key\s*[:=]\s*)([A-Za-z0-9/+=]{16,})", r"\1***", text)
        return text
    except Exception:
        return text



"""High-performance async GitHub API client with connection pooling."""
import aiohttp
import asyncio
import time
import json
from typing import Optional, Dict, Any, List, Tuple
from exc_analyzer.print_utils import Print
from exc_analyzer.config import load_key
from exc_analyzer.errors import ExcAnalyzerError
from exc_analyzer.i18n import t
MAX_CONCURRENT_REQUESTS = 10
RATE_LIMIT_THRESHOLD = 100
class AsyncGitHubAPI:
    """High-performance async GitHub API client with rate limit handling."""
    def __init__(self, token: str):
        self.token = token
        self.headers = {
            "Authorization": f"token {token}",
            "Accept": "application/vnd.github.v3+json"
        }
        self.graphql_headers = {
            "Authorization": f"token {token}",
            "Content-Type": "application/json",
            "Accept": "application/vnd.github.graphql"
        }
        self.remaining_quota = None
        self.reset_time = None
        self.session: Optional[aiohttp.ClientSession] = None
        self.semaphore = asyncio.Semaphore(MAX_CONCURRENT_REQUESTS)
        self._request_count = 0
    async def __aenter__(self):
        """Enter async context manager."""
        connector = aiohttp.TCPConnector(limit=MAX_CONCURRENT_REQUESTS, limit_per_host=5)
        self.session = aiohttp.ClientSession(connector=connector)
        return self
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Exit async context manager and close session."""
        if self.session:
            await self.session.close()
    async def _request(self, method: str, url: str, **kwargs) -> Tuple[Any, int]:
        """Execute HTTP request with rate limit handling and automatic retry."""
        if not self.session:
            raise ExcAnalyzerError("Session not initialized. Use 'async with' context manager.")
        async with self.semaphore:
            max_retries = 3
            for attempt in range(max_retries):
                try:
                    async with self.session.request(
                        method, url, timeout=aiohttp.ClientTimeout(total=30), **kwargs
                    ) as resp:
                        self.remaining_quota = int(resp.headers.get("X-RateLimit-Remaining", 0))
                        self.reset_time = int(resp.headers.get("X-RateLimit-Reset", 0))
                        if resp.status == 403 and self.remaining_quota == 0:
                            wait_time = max(1, self.reset_time - int(time.time()))
                            if wait_time <= 60:
                                await asyncio.sleep(wait_time + 1)
                                continue
                            else:
                                raise ExcAnalyzerError(
                                    "Rate limit exceeded. Please try again later."
                                )
                        if resp.status >= 500:
                            if attempt < max_retries - 1:
                                await asyncio.sleep(2 ** attempt)
                                continue
                            raise ExcAnalyzerError(f"Server error: {resp.status}")
                        try:
                            data = await resp.json()
                        except (json.JSONDecodeError, aiohttp.ContentTypeError):
                            data = {} if resp.status >= 400 else await resp.text()
                        self._request_count += 1
                        return data, resp.status
                except asyncio.TimeoutError:
                    if attempt < max_retries - 1:
                        await asyncio.sleep(1)
                        continue
                    raise ExcAnalyzerError("Request timeout")
                except aiohttp.ClientError as e:
                    if attempt < max_retries - 1:
                        await asyncio.sleep(1)
                        continue
                    raise ExcAnalyzerError(f"Connection error: {str(e)}")
            raise ExcAnalyzerError("Max retries exceeded")
    async def graphql_query(self, query_body: Dict[str, Any]) -> Dict:
        """Execute GraphQL query with error handling."""
        data, status = await self._request(
            "POST",
            "https://api.github.com/graphql",
            headers=self.graphql_headers,
            json=query_body
        )
        if status not in (200, 201):
            raise ExcAnalyzerError(f"GraphQL error: HTTP {status}")
        if isinstance(data, dict):
            if "errors" in data:
                errors = data.get("errors", [])
                error_msg = errors[0].get("message", "Unknown error") if errors else "Unknown error"
                raise ExcAnalyzerError(f"GraphQL error: {error_msg}")
        return data.get("data", {})
    async def fetch_json(self, url: str, params: Optional[Dict] = None) -> Dict:
        """Fetch JSON from REST API endpoint."""
        data, status = await self._request("GET", url, headers=self.headers, params=params)
        if status == 404:
            raise ExcAnalyzerError("Resource not found")
        if status == 401:
            raise ExcAnalyzerError("Authentication failed")
        if status >= 400:
            raise ExcAnalyzerError(f"API error: {status}")
        return data if isinstance(data, dict) else {}
    async def fetch_multiple(self, urls: List[str]) -> List[Dict]:
        """Fetch multiple URLs concurrently."""
        tasks = [self.fetch_json(url) for url in urls]
        results = await asyncio.gather(*tasks, return_exceptions=True)
        return [r for r in results if not isinstance(r, Exception)]
    async def fetch_paginated(
        self, url: str, per_page: int = 100, max_pages: Optional[int] = None
    ) -> List[Dict]:
        """Fetch paginated results with concurrent page fetching."""
        first_page, status = await self._request(
            "GET", url, headers=self.headers, params={"per_page": per_page, "page": 1}
        )
        if status != 200:
            return []
        if not isinstance(first_page, list):
            return [first_page] if first_page else []
        results = list(first_page)
        page_tasks = []
        for page in range(2, (max_pages or 100) + 1):
            if len(page_tasks) >= 5:
                done, pending = await asyncio.wait(page_tasks, return_when=asyncio.FIRST_COMPLETED)
                page_tasks = list(pending)
                for task in done:
                    page_data = await task
                    if page_data and isinstance(page_data, list):
                        results.extend(page_data)
                    else:
                        return results  
            task = asyncio.create_task(
                self._fetch_page(url, per_page, page)
            )
            page_tasks.append(task)
        for task in page_tasks:
            page_data = await task
            if page_data and isinstance(page_data, list) and len(page_data) > 0:
                results.extend(page_data)
            else:
                break
        return results
    async def _fetch_page(self, url: str, per_page: int, page: int) -> Optional[List]:
        """Fetch a single page of results."""
        try:
            data, status = await self._request(
                "GET", url, headers=self.headers, params={"per_page": per_page, "page": page}
            )
            return data if isinstance(data, list) else None
        except Exception:
            return None
    def get_quota_info(self) -> str:
        """Get current rate limit quota information."""
        if self.remaining_quota is not None:
            return t("commands.shared.quota_remaining", count=self.remaining_quota)
        return t("commands.shared.quota_unavailable")
async def get_async_client(token: Optional[str] = None) -> AsyncGitHubAPI:
    """Get async GitHub API client."""
    if not token:
        token = load_key()
        if not token:
            raise ExcAnalyzerError(f"{t('commands.shared.api_key_missing')} {t('commands.shared.use_key_cmd')}")
    return AsyncGitHubAPI(token)
def create_async_context(token: Optional[str] = None):
    """Create async context manager for API client."""
    if not token:
        token = load_key()
        if not token:
            raise ExcAnalyzerError(f"{t('commands.shared.api_key_missing')} {t('commands.shared.use_key_cmd')}")
    return AsyncGitHubAPI(token)



import sys
import os
import ctypes
from exc_analyzer.constants import CONFIG_DIR, LOG_FILE
VERBOSE = False
_OUTPUT_FILE = None
_OUTPUT_STATS = {'lines': 0, 'bytes': 0}
_WRITE_ERROR_SHOWN = False
def _enable_windows_vt_mode():
    """Enable Virtual Terminal Processing on Windows for ANSI/\r support."""
    if os.name != 'nt':
        return
    try:
        kernel32 = ctypes.windll.kernel32
        hStdOut = kernel32.GetStdHandle(-11)
        mode = ctypes.c_ulong()
        if not kernel32.GetConsoleMode(hStdOut, ctypes.byref(mode)):
            return
        if not (mode.value & 0x0004):
            kernel32.SetConsoleMode(hStdOut, mode.value | 0x0004)
    except Exception:
        pass
_enable_windows_vt_mode()
def supports_color():
    plat = sys.platform
    if plat == 'win32':
        return hasattr(sys.stdout, 'isatty') and sys.stdout.isatty()
    supported_platform = plat != 'Pocket PC' and (plat != 'win32' or 'ANSICON' in os.environ or 'WT_SESSION' in os.environ or 'TERM' in os.environ)
    is_a_tty = hasattr(sys.stdout, 'isatty') and sys.stdout.isatty()
    return supported_platform and is_a_tty
COLOR_ENABLED = supports_color()
def colorize(text, color_code):
    if COLOR_ENABLED:
        return f"\033[{color_code}m{text}\033[0m"
    return text
def _strip_ansi(text):
    """Remove ANSI color codes from text."""
    import re
    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
    osc_escape = re.compile(r'\x1B\].*?(?:\x07|\x1B\\)')
    text = ansi_escape.sub('', text)
    return osc_escape.sub('', text)
def clear_screen():
    """Clear the terminal screen."""
    os.system('cls' if os.name == 'nt' else 'clear')
def set_output_file(filepath):
    """Set output file for saving CLI output."""
    global _OUTPUT_FILE, _OUTPUT_STATS, _WRITE_ERROR_SHOWN
    from exc_analyzer.i18n import t
    if filepath:
        try:
            _OUTPUT_FILE = open(filepath, 'w', encoding='utf-8')
            _OUTPUT_STATS = {'lines': 0, 'bytes': 0}
            _WRITE_ERROR_SHOWN = False
        except PermissionError:
            from exc_analyzer.logging_utils import log
            Print.warn(t("commands.shared.permission_denied"))
            log(f"Output file permission error: {filepath}")
            _OUTPUT_FILE = None
        except Exception as e:
            from exc_analyzer.logging_utils import log
            error_msg = t("commands.shared.output_file_error", filepath=filepath, error=str(e))
            Print.warn(error_msg)
            log(f"Output file error: {filepath}: {e}")
            _OUTPUT_FILE = None
    else:
        _OUTPUT_FILE = None
def close_output_file():
    """Close output file if it's open."""
    global _OUTPUT_FILE, _OUTPUT_STATS
    from exc_analyzer.i18n import t
    from exc_analyzer.helpers import format_size
    if _OUTPUT_FILE:
        try:
            filepath = _OUTPUT_FILE.name
            lines_count = _OUTPUT_STATS['lines']
            bytes_count = _OUTPUT_STATS['bytes']
            _OUTPUT_FILE.close()
            _OUTPUT_FILE = None  
            size_str = format_size(bytes_count)
            Print.success(t("output.file_closed_stats", 
                          filepath=filepath, 
                          lines=lines_count, 
                          size=size_str))
        except Exception:
            pass
        _OUTPUT_FILE = None
def _write_output(text):
    """Write to output file without colors."""
    global _OUTPUT_FILE, _OUTPUT_STATS, _WRITE_ERROR_SHOWN
    if _OUTPUT_FILE:
        try:
            clean_text = _strip_ansi(str(text))
            _OUTPUT_FILE.write(clean_text + '\n')
            _OUTPUT_FILE.flush()
            _OUTPUT_STATS['lines'] += 1
            _OUTPUT_STATS['bytes'] += len(clean_text) + 1  
        except Exception as e:
            if not _WRITE_ERROR_SHOWN:
                from exc_analyzer.i18n import t
                Print.warn(t("commands.shared.output_write_error", error=str(e)))
                _WRITE_ERROR_SHOWN = True
            from exc_analyzer.logging_utils import log
            log(f"Output write error: {e}")
def safe_print(text='', **kwargs):
    """Print text with Unicode encoding error handling."""
    try:
        print(text, **kwargs)
    except UnicodeEncodeError:
        try:
            encoded = text.encode('utf-8', errors='replace').decode('utf-8')
            print(encoded, **kwargs)
        except Exception:
            encoded_ascii = str(text).encode('ascii', errors='replace').decode('ascii')
            print(encoded_ascii, **kwargs)
    _write_output(text)
class Print:
    @staticmethod
    def success(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.success")
        colored_msg = colorize(f"[{prefix}] {msg}", '92')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def error(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.error")
        colored_msg = colorize(f"[{prefix}] {msg}", '91')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def warn(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.warn")
        colored_msg = colorize(f"[{prefix}] {msg}", '93')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def info(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.info")
        colored_msg = colorize(f"[{prefix}] {msg}", '96')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def action(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.action")
        colored_msg = colorize(f"[{prefix}] {msg}", '90')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def critical_error(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.critical_error")
        colored_msg = colorize(f"[{prefix}] {msg}", '1;91')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def cancelled(msg):
        from exc_analyzer.i18n import t
        prefix = t("log_levels.cancelled")
        colored_msg = colorize(f"[{prefix}] {msg}", '90')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"[{prefix}] {msg}")
    @staticmethod
    def link(url):
        colored_msg = colorize(url, '94')
        try:
            print(colored_msg)
        except UnicodeEncodeError:
            print(colored_msg.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(url)
    @staticmethod
    def hyperlink(text, url):
        """Print clickable hyperlink (OSC 8) with shortened text."""
        colored_text = colorize(text, '33')
        if COLOR_ENABLED:
            link_seq = f"\033]8;;{url}\033\\{colored_text}\033]8;;\033\\"
        else:
            link_seq = f"{text} ({url})"
        try:
            print(link_seq)
        except UnicodeEncodeError:
            print(link_seq.encode('utf-8', errors='replace').decode('utf-8'))
        _write_output(f"{text} ({url})")
    @staticmethod
    def colorize(text, color_code):
        return f"\033[{color_code}m{text}\033[0m"
def print_success(msg): Print.success(msg)
def print_error(msg): Print.error(msg)
def print_warning(msg): Print.warn(msg)
def print_info(msg): Print.info(msg)
def print_action(msg): Print.action(msg)
def print_action(msg): Print.action(msg)
def print_cancelled(msg): Print.cancelled(msg)
def print_critical_error(msg): Print.critical_error(msg)
def set_verbose(v: bool):
    """Set verbose flag for printing/logging."""
    global VERBOSE
    VERBOSE = bool(v)
_original_print = print
def _print_wrapper(*args, **kwargs):
    """Wrapper for print() that also writes to output file."""
    _original_print(*args, **kwargs)
    if _OUTPUT_FILE:
        try:
            text = ' '.join(str(arg) for arg in args)
            clean_text = _strip_ansi(text)
            _OUTPUT_FILE.write(clean_text + '\n')
            _OUTPUT_FILE.flush()
        except Exception:
            pass



import os
HOME_DIR = os.path.expanduser("~")
CONFIG_DIR = os.path.join(HOME_DIR, ".exc")
LOG_FILE = os.path.join(CONFIG_DIR, "exc.log")
CLIENT_ID = "Ov23li5ckngNd4fu4gO1"
DEVICE_CODE_URL = "https://github.com/login/device/code"
ACCESS_TOKEN_URL = "https://github.com/login/oauth/access_token"
AUTH_SCOPES = "repo read:org read:user user:email workflow"



"""Commit anomaly detection with improved visual output."""
from ..print_utils import Print, _write_output, safe_print
from ..api import api_get, get_auth_header
from ..helpers import format_friendly_date
from ..spinner import spinner
from ..i18n import t
def cmd_commit_anomaly(args):
    if not args.repo:
        Print.error(t("commands.shared.missing_repo", default="Missing required argument: <owner/repo>"))
        _write_output(f"\n{t('commands.commit_anomaly.usage')}")
        return
    headers = get_auth_header()
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        print()
        _write_output("")
        Print.error(t("commands.shared.invalid_repo_format"))
        print()
        return
    with spinner(t("commands.commit_anomaly.analyzing", repo=repo), color='96'):
        url = f"https://api.github.com/repos/{repo}/commits?per_page=30"
        commits, _ = api_get(url, headers)
    print()
    _write_output("")
    risky = []
    SUSPICIOUS = ["fix bug", "temp", "test", "remove security", "debug", "hack", "bypass", "password", "secret"]
    for c in commits:
        msg = c['commit']['message'].lower()
        original_msg = c['commit']['message']
        matched_words = [word for word in SUSPICIOUS if word in msg]
        if matched_words:
            c_date = format_friendly_date(c['commit']['author']['date'], include_relative=True)
            author = c['commit']['author']['name']
            sha = c['sha'][:7]
            risky.append({
                'sha': sha,
                'message': original_msg,
                'author': author,
                'date': c_date,
                'matched': matched_words
            })
    _print_anomalies(repo, risky)
def _print_anomalies(repo, risky):
    """Print commit anomalies in a visual, user-friendly format."""
    safe_print("")
    _write_output("")
    sep = "═" * 60
    safe_print(Print.colorize(sep, '96'))
    _write_output(sep)
    header_text = f"  {t('commands.commit_anomaly.header')}: {repo}"
    safe_print(Print.colorize(header_text, '97'))
    _write_output(header_text)
    safe_print(Print.colorize(sep, '96'))
    _write_output(sep)
    safe_print("")
    _write_output("")
    if not risky:
        msg = f"  [OK] {t('commands.commit_anomaly.none_found')}"
        safe_print(Print.colorize(msg, '92'))
        _write_output(msg)
        safe_print("")
        _write_output("")
        sep = "═" * 60
        safe_print(Print.colorize(sep, '96'))
        _write_output(sep)
        safe_print("")
        _write_output("")
        return
    count_text = t("commands.commit_anomaly.found_count", count=len(risky))
    safe_print(Print.colorize(f"  [!] {count_text}", '93'))
    _write_output(f"  [!] {count_text}")
    safe_print("")
    _write_output("")
    for i, commit in enumerate(risky, 1):
        commit_header = f"  [{i}] Commit: {commit['sha']}"
        safe_print(Print.colorize(commit_header, '93'))
        _write_output(commit_header)
        msg_lines = commit['message'].split('\n')
        msg_first = msg_lines[0]
        if len(msg_first) > 65:
            msg_first = msg_first[:62] + "..."
        msg_line = f"      Message: {msg_first}"
        safe_print(Print.colorize(msg_line, '97'))
        _write_output(msg_line)
        author_line = f"      Author:  {commit['author']}"
        safe_print(Print.colorize(author_line, '96'))
        _write_output(author_line)
        date_line = f"      Date:    {commit['date']}"
        safe_print(Print.colorize(date_line, '96'))
        _write_output(date_line)
        keywords = ", ".join(commit['matched'])
        keywords_line = f"      {t('commands.commit_anomaly.keywords')}: {keywords}"
        safe_print(Print.colorize(keywords_line, '91'))
        _write_output(keywords_line)
        if i < len(risky):
            safe_print("")
            _write_output("")
    safe_print("")
    _write_output("")
    safe_print(Print.colorize(sep, '96'))
    _write_output(sep)
    safe_print("")
    _write_output("")



import requests
import re
import csv
import json
import time
import math
from collections import Counter
from urllib.parse import quote, unquote
from ..print_utils import Print, _write_output
from ..api import api_get, get_auth_header
from ..i18n import t
from .dork_presets import PRESETS, get_preset_choices
from ..helpers import TablePrinter, _truncate, truncate_visual, get_visual_width
def cmd_dork_scan(args):
    if args.list_presets:
        _list_presets()
        return
    queries = []
    if args.query:
        queries.extend(args.query)
    if args.preset:
        if args.preset in PRESETS:
            queries.extend(PRESETS[args.preset])
        else:
            Print.error(t("commands.dork_scan.invalid_preset", preset=args.preset))
            return
    if not queries:
         Print.error(t("commands.shared.missing_query"))
         return
    headers = get_auth_header()
    num = args.num or 10
    if num > 100:
        num = 100
    full_query_str = " ".join(queries) 
    display_query = _truncate(full_query_str, 50)
    Print.info(t("commands.dork_scan.searching", query=display_query, num=num))
    _write_output("")
    search_term = ""
    processed_queries = [_smart_quote_query(q) for q in queries]
    if args.preset:
        search_term = " OR ".join(processed_queries)
    else:
        search_term = " ".join(processed_queries)
    if len(search_term) > 256:
        Print.warn(t("commands.dork_scan.long_query_warning"))
    from ..spinner import spinner
    url = f"https://api.github.com/search/code?q={quote(search_term)}&per_page={num}"
    data = {}
    with spinner(t("commands.dork_scan.searching_spinner"), color='96'):
        try:
            data, _ = api_get(url, headers)
        except Exception as e:
            Print.error(str(e))
            return
    items = data.get('items', [])
    if not items:
        Print.warn(t("commands.dork_scan.none_found"))
        return
    results = []
    if args.verify:
        Print.info(t("commands.dork_scan.verifying_count", count=len(items)))
        with spinner(t("commands.dork_scan.verifying_spinner"), color='93'):
            for item in items:
                status, severity = _verify_content(item, headers)
                results.append({
                    'repo': item['repository']['full_name'],
                    'path': item['path'],
                    'url': item['html_url'],
                    'status': status,
                    'severity': severity
                })
                time.sleep(0.1)
        clean_results = [r for r in results if r.get('severity', 0) == 0]
        risky_results = [r for r in results if r.get('severity', 0) > 0]
        risky_results.sort(key=lambda x: x['severity'], reverse=True)
        filtered_count = len(clean_results)
        final_results = risky_results
    else:
        for item in items:
            results.append({
                'repo': item['repository']['full_name'],
                'path': item['path'],
                'url': item['html_url'],
                'status': t("commands.dork_scan.status.unverified"),
                'severity': 0 
            })
        final_results = results
        filtered_count = 0
    _print_results(final_results, verify_enabled=args.verify)
    if filtered_count > 0:
        Print.info(t("commands.dork_scan.filtered_count", count=filtered_count))
    if args.export:
        _export_results(results, args.export)
def _verify_content(item, headers):
    """
    Fetch file content and check for obvious patterns.
    This is a 'lite' verification.
    """
    try:
        html_url = item.get('html_url', '')
        if 'github.com' in html_url and '/blob/' in html_url:
            raw_url = html_url.replace('github.com', 'raw.githubusercontent.com').replace('/blob/', '/')
            resp = requests.get(raw_url)
            if resp.status_code == 200:
                content = resp.text
                return _analyze_content(content)
    except Exception:
        pass
    return t("commands.dork_scan.status.check_failed"), 0
def _analyze_content(content):
    """
    Context-aware content analysis with false positive reduction.
    Returns (verification_status, severity_score)
    Severity: 3 (High/Verified), 2 (Suspicious), 1 (Potential), 0 (Clean/Safe)
    """
    lower_content = content.lower()
    example_indicators = [
        'example', 'sample', 'demo', 'test', 'placeholder', 'dummy',
        'fake', 'mock', 'your_key_here', 'replace_with', 'insert_here',
        'todo', 'fixme', 'xxx', 'yyy', 'zzz', '12345', 'abcdef'
    ]
    doc_indicators = ['readme', 'example', '.md', 'documentation', 'tutorial', 'guide']
    is_likely_doc = any(ind in lower_content[:500] for ind in doc_indicators)
    secret_patterns = {
        'AWS Access Key': r'(?<![A-Z0-9])AKIA[0-9A-Z]{16}(?![A-Z0-9])',
        'AWS Secret Key': r'(?i)aws.{0,20}secret.{0,20}[\'"][0-9a-zA-Z/+=]{40}[\'"]',
        'GitHub Token (Classic)': r'ghp_[a-zA-Z0-9]{36}',
        'GitHub Token (Fine-grained)': r'github_pat_[a-zA-Z0-9]{22}_[a-zA-Z0-9]{59}',
        'Slack Token': r'xox[baprs]-[0-9]{10,13}-[0-9]{10,13}-[a-zA-Z0-9]{24,32}',
        'Google API Key': r'AIza[0-9A-Za-z\-_]{35}',
        'Stripe Secret Key': r'sk_live_[0-9a-zA-Z]{24,}',
        'Private Key Header': r'-----BEGIN (RSA|DSA|EC|OPENSSH) PRIVATE KEY-----',
        'Generic High-Value Secret': r'(?i)(api[_-]?key|secret[_-]?key|auth[_-]?token|access[_-]?token)\s*[:=]\s*[\'"][A-Za-z0-9_\-/+=]{16,}[\'"]'
    }
    for secret_type, pattern in secret_patterns.items():
        matches = list(re.finditer(pattern, content))
        for match in matches:
            matched_text = match.group(0)
            start = max(0, match.start() - 100)
            end = min(len(content), match.end() + 100)
            context = content[start:end]
            if _is_likely_example_or_comment(context, matched_text, example_indicators):
                continue
            if secret_type in ['AWS Access Key', 'GitHub Token (Classic)', 'GitHub Token (Fine-grained)']:
                if _calculate_entropy(matched_text) < 3.5:
                    continue  
            return f"{t('commands.dork_scan.status.verified')} ({secret_type})", 3
    suspicious_patterns = [
        r'password\s*[:=]\s*[\'"][^\'"]{8,}[\'"]',
        r'token\s*[:=]\s*[\'"][^\'"]{20,}[\'"]',
        r'-----BEGIN .+ PRIVATE KEY-----'
    ]
    for pat in suspicious_patterns:
        if re.search(pat, content, re.IGNORECASE):
            matches = list(re.finditer(pat, content, re.IGNORECASE))
            for match in matches:
                start = max(0, match.start() - 100)
                end = min(len(content), match.end() + 100)
                context = content[start:end]
                if not _is_likely_example_or_comment(context, match.group(0), example_indicators):
                    return t("commands.dork_scan.status.suspicious"), 2
    if len(content) < 50:
        return t("commands.dork_scan.status.suspicious_small"), 1
    if is_likely_doc:
        return t("commands.dork_scan.status.likely_example"), 1
    return t("commands.dork_scan.status.no_secrets"), 0
def _is_likely_example_or_comment(context, matched_text, example_indicators):
    """Check if the match is likely in an example or comment."""
    context_lower = context.lower()
    for indicator in example_indicators:
        if indicator in context_lower:
            return True
    lines = context.split('\n')
    for line in lines:
        if matched_text in line:
            stripped = line.strip()
            if stripped.startswith('#'):
                return True
            if stripped.startswith('//') or '/*' in line or '*/' in line:
                return True
            if '<!--' in line or '-->' in line:
                return True
    return False
def _calculate_entropy(text):
    """Calculate Shannon entropy of a string."""
    if not text:
        return 0
    counter = Counter(text)
    length = len(text)
    entropy = 0
    for count in counter.values():
        probability = count / length
        entropy -= probability * math.log2(probability)
    return entropy
def _print_results(results, verify_enabled=False):
    columns = [
        {'header': t("commands.dork_scan.table_header.repo"), 'width': 25, 'color': '94'},
        {'header': t("commands.dork_scan.table_header.file"), 'width': None, 'color': '97'}, 
        {'header': t("commands.dork_scan.table_header.link"), 'width': 35, 'color': '33'}
    ]
    if verify_enabled:
        columns.append({'header': t("commands.dork_scan.table_header.status"), 'width': 20, 'color': '92'})
    safe_print("")
    printer = TablePrinter(columns)
    printer.print_header()
    w_repo = columns[0]['_actual_width']
    w_file = columns[1]['_actual_width']
    w_link = columns[2]['_actual_width']
    w_stat = columns[3]['_actual_width'] if verify_enabled else 0
    for i, res in enumerate(results):
        display_repo = _truncate(res['repo'], w_repo)
        display_file = smart_truncate_path(res['path'], w_file)
        display_link = shorten_display_url(res['url'], w_link)
        row = [display_repo, display_file, (display_link, res['url'])]
        if verify_enabled:
            row.append(_truncate(res['status'], w_stat))
        style = "2;" if i % 2 == 1 else ""
        printer.print_row(row, style_prefix=style)
    safe_print("")
    Print.info(t("commands.dork_scan.total", count=len(results)))
    Print.warn(t("commands.dork_scan.disclaimer"))
    safe_print("")
def _export_results(results, filepath):
    """Export results to JSON or CSV"""
    try:
        if filepath.lower().endswith('.json'):
            with open(filepath, 'w', encoding='utf-8') as f:
                json.dump(results, f, indent=2, ensure_ascii=False)
        else:
            if not filepath.lower().endswith('.csv'):
                filepath += '.csv'
            with open(filepath, 'w', newline='', encoding='utf-8') as f:
                writer = csv.writer(f)
                header = ['Repository', 'Path', 'URL', 'Status']
                writer.writerow(header)
                for r in results:
                    writer.writerow([r['repo'], r['path'], r['url'], r['status']])
        Print.success(t("commands.dork_scan.export_success", path=filepath))
    except Exception as e:
        Print.error(t("commands.dork_scan.export_error", error=str(e)))
def _list_presets():
    Print.info(t("commands.dork_scan.presets_list_header"))
    for k, v in PRESETS.items():
        examples = ", ".join(v[:3])
        if len(v) > 3:
            examples += "..."
        print(f"  - {k:<10} : {examples}")
    print()
def shorten_display_url(full_url, max_len):
    try:
       full_url = unquote(full_url)
    except:
       pass
    if "/blob/" not in full_url:
        return _truncate(full_url, max_len)
    try:
        parts = full_url.split("/blob/")
        rest = parts[1]
        sha_parts = rest.split("/", 1)
        filename = sha_parts[1].split("/")[-1] if len(sha_parts) > 1 else "file"
        prefix = "github.com/.../"
        available = max_len - len(prefix)
        if len(filename) > available:
            filename = _truncate(filename, available)
        return f"{prefix}{filename}"
    except:
         return _truncate(full_url, max_len)
def smart_truncate_path(path, max_width):
    """Truncate path with priority to filename"""
    visual_len = get_visual_width(path)
    if visual_len <= max_width:
        return path
    parts = path.split('/')
    if len(parts) > 1:
        filename = parts[-1]
        fn_len = get_visual_width(filename)
        if fn_len >= max_width:
             return truncate_visual(filename, max_width)
        rem = max_width - fn_len - 4 
        if rem > 3:
            folder = parts[0]
            truncated_folder = truncate_visual(folder, rem)
            return f"{truncated_folder}/.../{filename}"
        else:
            return f".../{filename}"
    return truncate_visual(path, max_width)
def _smart_quote_query(query):
    """
    Intelligently quote the query if it contains spaces but no search modifiers.
    Enables logic like:
    - Linus Torvalds -> "Linus Torvalds" (Exact match)
    - filename:test.py -> filename:test.py (Modifier, no quote)
    - repo:x error -> repo:x error (Contains modifier, no quote)
    """
    if query.startswith('"') and query.endswith('"'):
        return query
    if " " not in query:
        return query
    modifiers = [
        'repo', 'user', 'org', 'path', 'filename', 'extension', 
        'size', 'pushed', 'created', 'language', 'topic', 
        'license', 'followers', 'fork', 'stars', 'is'
    ]
    for mod in modifiers:
        if f"{mod}:" in query:
            return query
    return f'"{query}"'
def safe_print(msg=""):
    print(msg)
    _write_output(msg)



"""Advanced user anomaly detection with risk scoring and pattern analysis."""
from ..print_utils import Print, _write_output, safe_print
from ..api import api_get, get_auth_header
from ..i18n import t
from collections import Counter
from datetime import datetime, timezone
import re
def cmd_user_anomaly(args):
    if not args.username:
        Print.error(t("commands.user_a.missing_username"))
        _write_output(f"\n{t('commands.user_anomaly.usage')}")
        return
    headers = get_auth_header()
    user = args.username.strip()
    safe_print("")
    Print.info(t("commands.user_anomaly.checking", user=user))
    safe_print("")
    from ..spinner import spinner
    with spinner(t("commands.user_anomaly.fetching_data"), color='96'):
        user_url = f"https://api.github.com/users/{user}"
        user_data, _ = api_get(user_url, headers)
        events_url = f"https://api.github.com/users/{user}/events/public?per_page=100"
        events, _ = api_get(events_url, headers)
        repos_url = f"https://api.github.com/users/{user}/repos?per_page=100"
        repos, _ = api_get(repos_url, headers)
    with spinner(t("commands.user_anomaly.analyzing"), color='93'):
        analysis = _analyze_user(user_data, events, repos)
    _print_analysis(user, analysis)
def _analyze_user(user_data, events, repos):
    """Comprehensive user analysis with anomaly detection."""
    anomalies = []
    risk_score = 0
    created_at = user_data.get('created_at', '')
    if created_at:
        account_age_days = (datetime.now(timezone.utc) - datetime.fromisoformat(created_at.replace('Z', '+00:00'))).days
        public_repos = user_data.get('public_repos', 0)
        if account_age_days < 30 and public_repos > 10:
            anomalies.append({
                'type': 'warning',
                'message': t("commands.user_anomaly.anomaly.new_account_high_activity", 
                           days=account_age_days, repos=public_repos)
            })
            risk_score += 20
        if account_age_days > 365 and public_repos == 0 and len(events) == 0:
            anomalies.append({
                'type': 'info',
                'message': t("commands.user_anomaly.anomaly.old_account_no_activity")
            })
            risk_score += 10
    followers = user_data.get('followers', 0)
    following = user_data.get('following', 0)
    if following > 0:
        ratio = followers / following
        if following > 1000 and ratio < 0.1:
            anomalies.append({
                'type': 'warning',
                'message': t("commands.user_anomaly.anomaly.suspicious_follow_ratio",
                           followers=followers, following=following)
            })
            risk_score += 25
    if repos:
        fork_count = sum(1 for r in repos if r.get('fork', False))
        fork_ratio = fork_count / len(repos) if len(repos) > 0 else 0
        if fork_ratio > 0.8 and len(repos) > 5:
            anomalies.append({
                'type': 'warning',
                'message': t("commands.user_anomaly.anomaly.high_fork_ratio",
                           ratio=int(fork_ratio * 100), forks=fork_count, total=len(repos))
            })
            risk_score += 15
    if events:
        hours = [int(e['created_at'][11:13]) for e in events if 'created_at' in e]
        hour_counts = Counter(hours)
        if hour_counts:
            most_common_hour, count = hour_counts.most_common(1)[0]
            if count > len(hours) * 0.5 and len(hours) > 10:
                anomalies.append({
                    'type': 'warning',
                    'message': t("commands.user_anomaly.anomaly.concentrated_activity",
                               count=count, hour=most_common_hour)
                })
                risk_score += 15
            night_activity = sum(hour_counts.get(h, 0) for h in range(0, 6))
            if night_activity > len(hours) * 0.7 and len(hours) > 10:
                anomalies.append({
                    'type': 'info',
                    'message': t("commands.user_anomaly.anomaly.night_activity",
                               percent=int((night_activity/len(hours))*100))
                })
                risk_score += 5
    profile_score = 0
    if user_data.get('name'): profile_score += 1
    if user_data.get('bio'): profile_score += 1
    if user_data.get('location'): profile_score += 1
    if user_data.get('email'): profile_score += 1
    if user_data.get('blog'): profile_score += 1
    if profile_score <= 1 and (followers > 100 or public_repos > 10):
        anomalies.append({
            'type': 'info',
            'message': t("commands.user_anomaly.anomaly.incomplete_profile")
        })
        risk_score += 10
    if events:
        event_types = Counter(e.get('type', 'Unknown') for e in events)
        if event_types:
            most_common_type, type_count = event_types.most_common(1)[0]
            if type_count > len(events) * 0.9 and len(events) > 20:
                anomalies.append({
                    'type': 'warning',
                    'message': t("commands.user_anomaly.anomaly.repetitive_events",
                               event_type=most_common_type, percent=int((type_count/len(events))*100))
                })
                risk_score += 20
    if events:
        push_events = [e for e in events if e.get('type') == 'PushEvent']
        if push_events:
            commit_messages = []
            for event in push_events:
                commits = event.get('payload', {}).get('commits', [])
                for commit in commits:
                    msg = commit.get('message', '')
                    if msg:
                        commit_messages.append(msg)
            if commit_messages:
                spam_patterns = ['update', 'fix', 'test', 'commit', 'change', 'modify']
                single_word_count = sum(1 for msg in commit_messages if len(msg.split()) <= 2)
                spam_word_count = sum(1 for msg in commit_messages 
                                     if any(pattern in msg.lower() for pattern in spam_patterns) 
                                     and len(msg.split()) <= 3)
                if len(commit_messages) > 10:
                    if single_word_count > len(commit_messages) * 0.7:
                        anomalies.append({
                            'type': 'warning',
                            'message': t("commands.user_anomaly.anomaly.low_quality_commits",
                                       percent=int((single_word_count/len(commit_messages))*100))
                        })
                        risk_score += 15
    if repos and len(repos) > 5:
        repo_names = [r.get('name', '') for r in repos]
        random_pattern_count = sum(1 for name in repo_names 
                                  if re.match(r'^[a-z]{8,}\d+$', name.lower()) or 
                                     re.match(r'^repo-?\d+$', name.lower()) or
                                     re.match(r'^test-?\d+$', name.lower()))
        if random_pattern_count > len(repos) * 0.5:
            anomalies.append({
                'type': 'warning',
                'message': t("commands.user_anomaly.anomaly.random_repo_names",
                           count=random_pattern_count, total=len(repos))
            })
            risk_score += 20
    if events and len(events) > 20:
        from collections import defaultdict
        events_by_day = defaultdict(int)
        for event in events:
            day = event.get('created_at', '')[:10]  
            if day:
                events_by_day[day] += 1
        if events_by_day:
            max_daily = max(events_by_day.values())
            avg_daily = sum(events_by_day.values()) / len(events_by_day)
            if max_daily > avg_daily * 5 and avg_daily > 1:
                anomalies.append({
                    'type': 'info',
                    'message': t("commands.user_anomaly.anomaly.activity_burst",
                               max_events=max_daily, avg_events=int(avg_daily))
                })
                risk_score += 10
    if repos and len(repos) > 5:
        languages = [r.get('language') for r in repos if r.get('language')]
        if languages:
            lang_counter = Counter(languages)
            most_common_lang, lang_count = lang_counter.most_common(1)[0]
            if lang_count > len(languages) * 0.9 and len(languages) > 10:
                anomalies.append({
                    'type': 'info',
                    'message': t("commands.user_anomaly.anomaly.single_language_focus",
                               language=most_common_lang, percent=int((lang_count/len(languages))*100))
                })
    if repos and len(repos) > 5:
        empty_repos = sum(1 for r in repos if r.get('size', 0) < 10)  
        if empty_repos > len(repos) * 0.7:
            anomalies.append({
                'type': 'warning',
                'message': t("commands.user_anomaly.anomaly.mostly_empty_repos",
                           empty=empty_repos, total=len(repos))
            })
            risk_score += 15
    risk_score = min(risk_score, 100)
    return {
        'user_data': user_data,
        'events': events,
        'repos': repos,
        'anomalies': anomalies,
        'risk_score': risk_score,
        'hour_counts': Counter([int(e['created_at'][11:13]) for e in events if 'created_at' in e]) if events else Counter()
    }
def _print_analysis(username, analysis):
    """Print formatted analysis results."""
    user_data = analysis['user_data']
    anomalies = analysis['anomalies']
    risk_score = analysis['risk_score']
    hour_counts = analysis['hour_counts']
    safe_print(Print.colorize("═" * 60, '96'))
    header = f"  {t('commands.user_anomaly.header', user=username)}"
    safe_print(Print.colorize(header, '1;97'))
    safe_print(Print.colorize("═" * 60, '96'))
    safe_print("")
    risk_level, risk_color = _get_risk_level(risk_score)
    score_text = f"  {t('commands.user_anomaly.risk_score')}: {risk_score}/100 ({risk_level})"
    safe_print(Print.colorize(score_text, risk_color))
    safe_print("")
    if anomalies:
        safe_print(Print.colorize(f"  {t('commands.user_anomaly.anomalies_detected')}:", '93'))
        safe_print("")
        for anomaly in anomalies:
            if anomaly['type'] == 'warning':
                Print.warn(f"  {anomaly['message']}")
            else:
                Print.info(f"  {anomaly['message']}")
        safe_print("")
    else:
        Print.success(f"  {t('commands.user_anomaly.no_anomalies')}")
        safe_print("")
    safe_print(Print.colorize(f"  {t('commands.user_anomaly.profile_summary')}:", '96'))
    safe_print("")
    created_at = user_data.get('created_at', '')
    if created_at:
        account_age_days = (datetime.now(timezone.utc) - datetime.fromisoformat(created_at.replace('Z', '+00:00'))).days
        account_age_years = account_age_days / 365
        age_str = f"{account_age_years:.1f} {t('time.years')}" if account_age_years >= 1 else f"{account_age_days} {t('time.days')}"
    else:
        age_str = t("commands.shared.unknown")
    safe_print(f"    {t('commands.user_anomaly.account_age')}: {age_str}")
    safe_print(f"    {t('commands.user_anomaly.total_repos')}: {user_data.get('public_repos', 0)}")
    safe_print(f"    {t('commands.user_anomaly.followers_following')}: {user_data.get('followers', 0)}/{user_data.get('following', 0)}")
    if analysis['repos']:
        fork_count = sum(1 for r in analysis['repos'] if r.get('fork', False))
        safe_print(f"    {t('commands.user_anomaly.fork_ratio')}: {fork_count}/{len(analysis['repos'])}")
    safe_print("")
    if hour_counts:
        safe_print(Print.colorize(f"  {t('commands.user_anomaly.activity_chart')}:", '96'))
        safe_print("")
        _print_activity_chart(hour_counts)
        safe_print("")
    safe_print(Print.colorize("═" * 60, '96'))
    safe_print("")
def _get_risk_level(score):
    """Get risk level label and color based on score."""
    if score < 30:
        return t("commands.user_anomaly.risk_low"), '92'  
    elif score < 60:
        return t("commands.user_anomaly.risk_medium"), '93'  
    else:
        return t("commands.user_anomaly.risk_high"), '91'  
def _print_activity_chart(hour_counts):
    """Print ASCII activity chart by time blocks."""
    blocks = {
        '00-06': sum(hour_counts.get(h, 0) for h in range(0, 6)),
        '06-12': sum(hour_counts.get(h, 0) for h in range(6, 12)),
        '12-18': sum(hour_counts.get(h, 0) for h in range(12, 18)),
        '18-24': sum(hour_counts.get(h, 0) for h in range(18, 24))
    }
    max_count = max(blocks.values()) if blocks.values() else 1
    bar_width = 30
    for time_range, count in blocks.items():
        bar_length = int((count / max_count) * bar_width) if max_count > 0 else 0
        bar = "█" * bar_length + "░" * (bar_width - bar_length)
        safe_print(f"    {time_range}: {bar} ({count})")



import os
from datetime import datetime
from exc_analyzer.constants import LOG_FILE
from exc_analyzer.print_utils import Print, VERBOSE
from exc_analyzer.utils import mask_sensitive
def log(msg):
    try:
        masked = mask_sensitive(str(msg))
    except Exception:
        masked = str(msg)
    if VERBOSE:
        Print.info(masked)
    try:
        log_dir = os.path.dirname(LOG_FILE)
        if not os.path.isdir(log_dir):
            os.makedirs(log_dir, exist_ok=True)
            try:
                os.chmod(log_dir, 0o700)
            except Exception:
                pass
        if os.path.isfile(LOG_FILE) and os.path.getsize(LOG_FILE) > 1024*1024:
            os.remove(LOG_FILE)
        with open(LOG_FILE, "a", encoding="utf-8") as f:
            f.write(f"[{datetime.now().isoformat()}] {masked}\n")
        try:
            os.chmod(LOG_FILE, 0o600)
        except Exception:
            pass
    except Exception as e:
        if VERBOSE:
            Print.warn(f"Log file error: {e}")



"""Optimized secret scanning with concurrent file processing."""
import asyncio
import re
from ..print_utils import print_info, print_warning, print_error, print_success, print_cancelled, safe_print
from ..async_api import create_async_context
from ..config import load_key
from ..i18n import t
SECRET_PATTERNS = {
    'AWS Key': r'AKIA[0-9A-Z]{16}',
    'GitHub Token': r'ghp_[a-zA-Z0-9]{36}',
    'SSH Private': r'-----BEGIN (RSA|OPENSSH) PRIVATE KEY-----',
    'API Key': r'(?i)(api|access)[_ -]?key["\']?[:=] ?["\'][0-9a-zA-Z]{20,40}'
}
async def _scan_secrets_async(repo: str, token: str, limit: int = 10, show_rate_limit: bool = False):
    """Async secret scanning with concurrent file downloads."""
    from ..print_utils import _write_output
    from ..spinner import spinner
    async with create_async_context(token) as client:
        try:
            with spinner(t("commands.scan_secrets.scanning", limit=limit, repo=repo), color='96'):
                commits_url = f"https://api.github.com/repos/{repo}/commits"
                commits = await client.fetch_paginated(commits_url, per_page=limit, max_pages=1)
            if not commits:
                print_warning(t("commands.scan_secrets.no_commits"))
                return
            found_secrets = []
            tasks = []
            for commit in commits[:limit]:
                commit_url = commit.get('url')
                if commit_url:
                    task = asyncio.create_task(_check_commit(client, commit, repo))
                    tasks.append(task)
            results = await asyncio.gather(*tasks, return_exceptions=True)
            for result in results:
                if isinstance(result, list):
                    found_secrets.extend(result)
            from ..helpers import format_friendly_date
            import shutil
            from datetime import datetime
            from ..print_utils import Print
            found_count = len(found_secrets)
            if found_secrets:
                safe_print("")
                _write_output("")
                from ..helpers import TablePrinter
                columns = [
                    {'header': t("commands.scan_secrets.table_header.type"), 'width': 15, 'color': '96'},   
                    {'header': t("commands.scan_secrets.table_header.date"), 'width': 10, 'color': '93'},   
                    {'header': t("commands.scan_secrets.table_header.author"), 'width': 18, 'color': '92'}, 
                    {'header': t("commands.scan_secrets.table_header.file"), 'width': None, 'color': '97'}, 
                ]
                printer = TablePrinter(columns)
                printer.print_header()
                for idx, secret in enumerate(found_secrets):
                    style = "2;" if idx % 2 == 1 else ""
                    d_raw = secret.get('date', 'N/A')
                    date_val = d_raw[:10] if isinstance(d_raw, str) and len(d_raw) >= 10 else str(d_raw)
                    row_data = [
                        secret.get('type', 'N/A'),
                        date_val,
                        secret.get('author', 'N/A'),
                        secret.get('file', 'N/A')
                    ]
                    printer.print_row(row_data, style_prefix=style)
                safe_print("")
                print_info(t("commands.scan_secrets.secret_found", count=found_count))
                safe_print("")
                _write_output("")
            else:
                safe_print("")
                print_success(t("commands.scan_secrets.none_found"))
                safe_print("")
            if show_rate_limit:
                safe_print("")
                _write_output("")
                print_info(client.get_quota_info())
                safe_print("")
                _write_output("")
        except Exception as e:
            print_error(f"Scan failed: {str(e)}")
async def _check_commit(client, commit: dict, repo: str) -> list:
    """Check a single commit for secrets."""
    secrets = []
    try:
        commit_url = commit.get('url')
        if not commit_url:
            return secrets
        commit_data = await client.fetch_json(commit_url)
        files = commit_data.get('files', [])
        sem = asyncio.Semaphore(10)
        file_tasks = []
        async def _check_with_sem(client, file, commit, repo):
            async with sem:
                return await _check_file(client, file, commit, repo)
        for file in files:
            if file.get('status') == 'added':
                task = asyncio.create_task(_check_with_sem(client, file, commit, repo))
                file_tasks.append(task)
        file_results = await asyncio.gather(*file_tasks, return_exceptions=True)
        for result in file_results:
            if isinstance(result, list):
                secrets.extend(result)
        return secrets
    except Exception:
        return secrets
async def _check_file(client, file: dict, commit: dict, repo: str) -> list:
    """Check a single file for secrets."""
    secrets = []
    try:
        content_url = file.get('raw_url')
        if not content_url:
            return secrets
        content = await client.fetch_json(content_url) if content_url.startswith('https://api') else None
        if not content:
            try:
                async with client.session.get(content_url, timeout=10) as resp:
                    if resp.status == 200:
                        content = await resp.text()
            except Exception:
                return secrets
        for secret_type, pattern in SECRET_PATTERNS.items():
            if isinstance(content, str) and re.search(pattern, content):
                secrets.append({
                    'type': secret_type,
                    'file': file.get('filename'),
                    'commit': commit.get('html_url'),
                    'date': commit.get('commit', {}).get('author', {}).get('date')
                })
                break
        return secrets
    except Exception:
        return secrets
def cmd_scan_secrets(args):
    """Scan recent commits for secrets - optimized with async processing."""
    from ..print_utils import _write_output
    if not args.repo:
        print_error(t("commands.shared.missing_repo"))
        _write_output(f"\n{t('commands.scan_secrets.usage')}")
        _write_output(t("commands.shared.usage_hint", command="scan-secrets", args="torvalds/linux -l 50"))
        return
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        safe_print("")
        _write_output("")
        print_error(t("commands.shared.invalid_repo_format"))
        safe_print("")
        return
    limit = args.limit if hasattr(args, 'limit') else 10
    if limit > 50:
        safe_print("")
        print_error(t("commands.scan_secrets.limit_exceeded", requested=limit, max=50))
        safe_print("")
        return
    if limit > 50:
        safe_print("")
        print_error(t("commands.scan_secrets.limit_exceeded", requested=limit, max=50))
        safe_print("")
        return
    token = load_key()
    if not token:
        safe_print("")
        _write_output("")
        print_error(t("commands.shared.api_key_missing"))
        print_info(t("commands.shared.use_key_cmd"))
        safe_print("")
        _write_output("")
        return
    try:
        safe_print("")
        _write_output("")
        show_rate = getattr(args, 'show_rate_limit', False)
        asyncio.run(_scan_secrets_async(repo, token, limit, show_rate))
    except KeyboardInterrupt:
        safe_print("")
        _write_output("")
        print_cancelled(t("commands.shared.scan_cancelled"))
    except Exception as e:
        print_error(t("commands.shared.error_occurred", error=str(e)))



"""Security score calculation with improved visual output."""
import requests
from ..print_utils import print_error, _write_output, safe_print, colorize, Print
from ..api import api_get, get_auth_header
from ..spinner import spinner
def cmd_security_score(args):
    from ..i18n import t
    if not args.repo:
        print_error(t("commands.shared.missing_repo"))
        _write_output(f"\n{t('commands.security_score.usage')}")
        _write_output(f"{t('commands.shared.example_header')} {t('commands.security_score.examples')[0]}")
        return
    headers = get_auth_header()
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        safe_print("")
        _write_output("")
        print_error(t("commands.shared.invalid_repo_format"))
        safe_print("")
        return
    with spinner(t("commands.security_score.calculating", repo=repo), color='96'):
        repo_url = f"https://api.github.com/repos/{repo}"
        repo_data, _ = api_get(repo_url, headers)
    safe_print("")
    _write_output("")
    score = 100
    issues = []  
    s_present = t("commands.security_score.status.present")
    s_missing = t("commands.security_score.status.missing")
    s_enabled = t("commands.security_score.status.enabled")
    s_disabled = t("commands.security_score.status.disabled")
    s_yes = t("commands.security_score.status.yes")
    s_no = t("commands.security_score.status.no")
    crit_license = t("commands.security_score.criteria.license")
    if not repo_data.get('license'):
        score -= 10
        issues.append((crit_license, t("commands.security_score.status.none"), -10, False))
    else:
        issues.append((crit_license, s_present, 0, True))
    crit_issues = t("commands.security_score.criteria.issues")
    if not repo_data.get('has_issues'):
        score -= 10
        issues.append((crit_issues, s_no, -10, False))
    else:
        issues.append((crit_issues, s_yes, 0, True))
    crit_wiki = t("commands.security_score.criteria.wiki")
    if not repo_data.get('has_wiki'):
        score -= 5
        issues.append((crit_wiki, s_no, -5, False))
    else:
        issues.append((crit_wiki, s_yes, 0, True))
    crit_projects = t("commands.security_score.criteria.projects")
    if not repo_data.get('has_projects'):
        score -= 5
        issues.append((crit_projects, s_no, -5, False))
    else:
        issues.append((crit_projects, s_yes, 0, True))
    crit_open = t("commands.security_score.criteria.open_issues")
    open_issues = repo_data.get('open_issues_count', 0)
    if open_issues > 50:
        score -= 10
        issues.append((crit_open, f"{open_issues}", -10, False))
    elif open_issues > 10:
        score -= 5
        issues.append((crit_open, f"{open_issues}", -5, False))
    else:
        issues.append((crit_open, f"{open_issues}", 0, True))
    crit_sec = t("commands.security_score.criteria.security_md")
    sec_url = f"https://api.github.com/repos/{repo}/contents/SECURITY.md"
    sec_resp = requests.get(sec_url, headers=headers)
    if sec_resp.status_code != 200:
        score -= 10
        issues.append((crit_sec, s_missing, -10, False))
    else:
        issues.append((crit_sec, s_present, 0, True))
    crit_bp = t("commands.security_score.criteria.branch_prot")
    default_branch = repo_data.get('default_branch')
    prot_url = f"https://api.github.com/repos/{repo}/branches/{default_branch}/protection"
    prot_resp = requests.get(prot_url, headers=headers)
    if prot_resp.status_code == 200:
        issues.append((crit_bp, s_enabled, 0, True))
    else:
        score -= 10
        issues.append((crit_bp, s_disabled, -10, False))
    crit_dep = t("commands.security_score.criteria.dependabot")
    dep_url = f"https://api.github.com/repos/{repo}/contents/.github/dependabot.yml"
    dep_resp = requests.get(dep_url, headers=headers)
    if dep_resp.status_code == 200:
        issues.append((crit_dep, s_present, 0, True))
    else:
        score -= 5
        issues.append((crit_dep, s_missing, -5, False))
    crit_cs = t("commands.security_score.criteria.code_scanning")
    scan_url = f"https://api.github.com/repos/{repo}/code-scanning/alerts"
    scan_resp = requests.get(scan_url, headers=headers)
    if scan_resp.status_code == 200:
        alerts = scan_resp.json()
        if isinstance(alerts, list) and len(alerts) > 0:
            score -= 10
            issues.append((crit_cs, f"{len(alerts)} {t('commands.security_score.status.open')}", -10, False))
        else:
            issues.append((crit_cs, "0", 0, True))
    else:
        issues.append((crit_cs, "N/A", 0, True))
    _print_security_score(repo, score, issues)
def _print_security_score(repo, score, issues):
    """Print security score in a visual, user-friendly format."""
    from ..i18n import t
    from ..helpers import TablePrinter
    if score >= 90:
        score_color = '92'  
        verdict = t("commands.security_score.verdict.excellent")
        verdict_color = '92'
    elif score >= 75:
        score_color = '93'  
        verdict = t("commands.security_score.verdict.good")
        verdict_color = '93'
    else:
        score_color = '91'  
        verdict = t("commands.security_score.verdict.weak")
        verdict_color = '91'
    safe_print("")
    safe_print(Print.colorize("═" * 60, '96'))
    header_text = f"  {t('commands.security_score.score_header')}: {repo}"
    safe_print(Print.colorize(header_text, '97'))
    safe_print(Print.colorize("═" * 60, '96'))
    safe_print("")
    score_text = f"    {score}/100"
    safe_print(Print.colorize(score_text, score_color))
    safe_print(Print.colorize(f"    {verdict}", verdict_color))
    safe_print("")
    columns = [
        {'header': t("commands.security_score.headers.criteria"), 'width': 35},
        {'header': t("commands.security_score.headers.status"), 'width': 25},
        {'header': t("commands.security_score.headers.impact"), 'width': 15, 'align': 'right'}
    ]
    printer = TablePrinter(columns)
    printer.print_header()
    for criteria, status, impact, passed in issues:
        if passed:
            color = '92' 
            impact_str = "" 
        else:
            color = '91' 
            impact_str = f"({impact})" if impact < 0 else ""
        printer.print_row([criteria, status, impact_str], color_override=color)
    safe_print("")
    safe_print(Print.colorize("═" * 60, '96'))
    safe_print("")



PRESETS = {
    "secrets": [
        "filename:.env",
        "filename:id_rsa",
        "filename:id_dsa",
        "filename:id_ed25519",
        "filename:*.pem",
        "filename:*.key",
        "filename:secrets.json",
        "filename:deployment-config.json"
    ],
    "config": [
        "filename:wp-config.php",
        "filename:config.php",
        "filename:config.json",
        "filename:settings.py",
        "filename:database.yml",
        "filename:settings.json",
        "filename:Web.config"
    ],
    "actions": [
        "filename:main.yml path:.github/workflows",
        "filename:deploy.yml path:.github/workflows",
        "filename:release.yml path:.github/workflows",
        "filename:ci.yml path:.github/workflows"
    ],
    "aws": [
        "filename:.aws/credentials",
        "filename:.aws/config",
        "AKIAIOSFODNN7EXAMPLE",
        "aws_access_key_id",
        "aws_secret_access_key"
    ],
    "azure": [
        "azure_storage_account",
        "azure_storage_access_key",
        "filename:azureProfile.json"
    ],
    "google": [
        "filename:client_secret.json",
        "filename:service_account.json",
        "GOOGLE_APPLICATION_CREDENTIALS"
    ]
}
def get_preset_choices():
    return list(PRESETS.keys())



import os
import requests
import time
import threading
import exc_analyzer
from datetime import datetime
from packaging.version import Version
from exc_analyzer.print_utils import Print
from exc_analyzer.config import load_key
from exc_analyzer import __version__ as local_version
from exc_analyzer.errors import ExcAnalyzerError
from exc_analyzer.utils import mask_sensitive
from typing import Optional, Tuple
import sys
import json
from exc_analyzer.constants import CLIENT_ID, DEVICE_CODE_URL, ACCESS_TOKEN_URL, AUTH_SCOPES
CACHE_TTL_SECONDS = 30
MAX_RATE_LIMIT_WAIT = 60
MAX_API_RETRIES = 3
_response_cache: dict = {}
def clear_response_cache():
    """Testing helper: resets the in-memory HTTP cache."""
    _response_cache.clear()
def _cache_key(url: str, params: Optional[dict]) -> Tuple[str, Tuple[Tuple[str, str], ...]]:
    if not params:
        return (url, tuple())
    return (url, tuple(sorted((str(k), str(v)) for k, v in params.items())))
def _get_cached_response(key):
    entry = _response_cache.get(key)
    if not entry:
        return None
    timestamp, payload = entry
    if time.time() - timestamp > CACHE_TTL_SECONDS:
        del _response_cache[key]
        return None
    return payload
def _store_cached_response(key, payload):
    _response_cache[key] = (time.time(), payload)
def _extract_rate_limit_wait(headers: dict) -> Optional[int]:
    if not headers:
        return None
    retry_after = headers.get('Retry-After')
    if retry_after:
        try:
            return max(0, int(float(retry_after)))
        except ValueError:
            pass
    reset = headers.get('X-RateLimit-Reset')
    if reset:
        try:
            reset_time = int(reset)
            wait_seconds = max(0, reset_time - int(time.time()))
            return wait_seconds
        except ValueError:
            pass
    remaining = headers.get('X-RateLimit-Remaining')
    if remaining == '0':
        return 5
    return None
def safe_get(url: str, headers: Optional[dict] = None, params: Optional[dict] = None, timeout: int = 10, max_bytes: int = 2_000_000, cacheable: bool = False):
    """Fetch URL safely with timeout and max content size.
    Returns a tuple: (text, headers, status_code).
    Does not raise on 403 so callers can inspect rate-limit headers.
    Raises for network errors or for responses with status >=500.
    """
    cache_key = None
    if cacheable:
        cache_key = _cache_key(url, params)
        cached = _get_cached_response(cache_key)
        if cached:
            return cached
    resp = requests.get(url, headers=headers, params=params, stream=True, timeout=timeout)
    status = getattr(resp, 'status_code', None)
    resp_headers = getattr(resp, 'headers', {}) or {}
    total = 0
    chunks = []
    for chunk in resp.iter_content(8192):
        if not chunk:
            continue
        total += len(chunk)
        if total > max_bytes:
            resp.close()
            raise ValueError("Content too large")
        chunks.append(chunk)
    content = b"".join(chunks)
    try:
        text = content.decode('utf-8', errors='ignore')
    except Exception:
        text = content.decode(errors='ignore')
    if status is not None and 500 <= status < 600:
        raise requests.HTTPError(f"Server error: {status}")
    payload = (text, resp_headers, status)
    if cacheable and status is not None and status < 400:
        _store_cached_response(cache_key, payload)
    return payload
def get_version_from_pyproject():
    try:
        init_path = os.path.join(os.path.dirname(__file__), "__init__.py")
        with open(init_path, "r", encoding="utf-8") as f:
            for line in f:
                if line.startswith("__version__"):
                    delim = '"' if '"' in line else "'"
                    version = line.split(delim)[1]
                    return version
    except Exception as e:
        from exc_analyzer.logging_utils import log
        log(f"Failed to read version from __init__.py: {e}")
    return None
def get_version_from_pypi():
    try:
        resp = requests.get("https://pypi.org/pypi/exc-analyzer/json", timeout=5)
        if resp.status_code == 200:
            return resp.json()["info"]["version"]
        else:
            Print.warn(f"PyPI responded with status code {resp.status_code}.")
    except Exception as e:
        Print.warn(f"Could not fetch version info from PyPI: {e}")
    return None
def notify_new_version():
    def _check():
        local_version = None
        try:
            local_version = exc_analyzer.__version__
        except Exception:
            local_version = get_version_from_pyproject()
        if local_version is None:
            return
        latest_version = get_version_from_pypi()
        if latest_version is None:
            return
        try:
            local_v = Version(local_version)
            latest_v = Version(latest_version)
            if local_v < latest_v:
                print("")
                Print.info(f"Update available: {latest_version}")
                Print.action("Use: pip install -U exc-analyzer")
        except Exception:
            pass
    t = threading.Thread(target=_check, daemon=True)
    t.start()
def api_get(url, headers, params=None, cacheable=True):
    from exc_analyzer.logging_utils import log
    attempt = 0
    while attempt < MAX_API_RETRIES:
        attempt += 1
        try:
            text, resp_headers, status = safe_get(
                url,
                headers=headers,
                params=params,
                timeout=12,
                cacheable=cacheable,
            )
        except requests.HTTPError as e:
            if attempt < MAX_API_RETRIES:
                wait = min(MAX_RATE_LIMIT_WAIT, 2 ** attempt)
                from exc_analyzer.i18n import t
                Print.warn(t("commands.shared.server_error_retry", attempt=attempt, max=MAX_API_RETRIES, wait=wait))
                time.sleep(wait)
                continue
            status = getattr(e.response, 'status_code', '?')
            log(f"HTTP error: {e}")
            Print.error(f"Failed to communicate with GitHub (HTTP {status}).")
            raise ExcAnalyzerError(f"HTTP error: {status}")
        from exc_analyzer.i18n import t
        if status == 403:
            wait_seconds = _extract_rate_limit_wait(resp_headers)
            if wait_seconds is not None and wait_seconds <= MAX_RATE_LIMIT_WAIT:
                Print.warn(t("commands.shared.rate_limit_reached"))
                Print.info(t("commands.shared.rate_limit_retry", wait=wait_seconds))
                time.sleep(max(1, wait_seconds))
                continue
            if resp_headers.get('X-RateLimit-Remaining') == '0':
                reset = resp_headers.get('X-RateLimit-Reset')
                if reset:
                    try:
                        readable_time = datetime.utcfromtimestamp(int(reset)).strftime('%Y-%m-%d %H:%M:%S UTC')
                        Print.info(t("commands.shared.rate_limit_resets", time=readable_time))
                    except Exception:
                        pass
            Print.warn(t("commands.shared.rate_limit_exceeded"))
            log("API rate limit exceeded.")
            raise ExcAnalyzerError("API rate limit exceeded")
        if status is not None and status >= 400:
            if status == 404:
                Print.error(t("commands.shared.resource_not_found"))
                Print.info(t("commands.shared.resource_check_typo"))
                print("")
            elif status == 401:
                Print.error(t("commands.shared.auth_failed"))
                Print.info(t("commands.shared.auth_check_creds"))
                print("")
            elif 500 <= status < 600:
                Print.error(t("commands.shared.server_error_http", status=status))
                print("")
            else:
                Print.error(t("commands.shared.invalid_response", status=status))
                print("")
            log(f"HTTP error: status={status} url={url}")
            raise ExcAnalyzerError(f"HTTP error: {status}")
        try:
            data = json.loads(text) if text else None
        except Exception as e:
            log(f"Failed to parse JSON response: {e}")
            raise ExcAnalyzerError("Invalid JSON received from API")
        return data, resp_headers
    raise ExcAnalyzerError("API request exceeded retry budget")
def get_auth_header():
    key = load_key()
    if not key:
        from exc_analyzer.i18n import t
        print("")
        Print.error(t("commands.shared.api_key_missing"))
        Print.info(t("commands.shared.use_key_cmd"))
        print("")
        sys.exit(1)
    return {
        "Authorization": f"token {key}",
        "Accept": "application/vnd.github.v3+json"
    }
def get_all_pages(url, headers, params=None):
    results = []
    page = 1
    while True:
        if params is None:
            params = {}
        params.update({'per_page': 100, 'page': page})
        data, resp_headers = api_get(url, headers, params, cacheable=False)
        if not isinstance(data, list):
            return data
        results.extend(data)
        if 'Link' in resp_headers:
            if 'rel="next"' not in resp_headers['Link']:
                break
        else:
            break
        page += 1
        time.sleep(0.15)
    return results
def fetch_github_user(key):
    """Fetch GitHub user information to validate key."""
    from exc_analyzer.i18n import t
    headers = {
        "Authorization": f"token {key}",
        "Accept": "application/vnd.github.v3+json"
    }
    try:
        response = requests.get("https://api.github.com/user", headers=headers, timeout=8)
        if response.status_code == 200:
            return response.json().get("login")
    except requests.RequestException as e:
        Print.error(t("commands.shared.key_validation_error", error=e))
    return None
def exchange_device_code():
    """Request a device code from GitHub."""
    headers = {"Accept": "application/json"}
    data = {"client_id": CLIENT_ID, "scope": AUTH_SCOPES}
    try:
        resp = requests.post(DEVICE_CODE_URL, headers=headers, data=data, timeout=10)
        resp.raise_for_status()
        return resp.json()
    except Exception as e:
        from exc_analyzer.logging_utils import log
        log(f"Device code exchange failed: {e}")
        return None
def poll_for_token(device_code: str, interval: int):
    """Poll GitHub for the access token."""
    from exc_analyzer.i18n import t
    max_duration = 900
    start_time = time.time()
    expires_at = start_time + max_duration
    headers = {"Accept": "application/json"}
    data = {
        "client_id": CLIENT_ID, 
        "device_code": device_code,
        "grant_type": "urn:ietf:params:oauth:grant-type:device_code"
    }
    while time.time() < expires_at:
        try:
            resp = requests.post(ACCESS_TOKEN_URL, headers=headers, data=data, timeout=10)
            if resp.status_code == 200:
                result = resp.json()
                if "access_token" in result:
                    return result["access_token"]
                error = result.get("error")
                if error == "authorization_pending":
                    pass
                elif error == "slow_down":
                    interval += 5
                elif error == "expired_token":
                    Print.error(t("commands.login.token_expired"))
                    return None
                elif error == "access_denied":
                    Print.error(t("commands.login.access_denied"))
                    return None
                else:
                    Print.warn(f"GitHub Error: {error}")
                    return None
            else:
                Print.warn(f"Polling HTTP Error: {resp.status_code}")
        except Exception as e:
            pass
        time.sleep(interval)
    Print.error(t("commands.login.timeout"))
    return None



"""GitHub Actions audit with improved visual output."""
import requests
import re
from ..print_utils import Print, _write_output, safe_print
from ..api import get_auth_header
from ..spinner import spinner
def cmd_actions_audit(args):
    from ..i18n import t
    if not args.repo:
        Print.error(t("commands.shared.missing_repo", default="Missing required argument: <owner/repo>"))
        _write_output(f"\n{t('commands.actions_audit.usage')}")
        _write_output(f"{t('commands.actions_audit.example')}")
        _write_output(f"\n{t('commands.actions_audit.description')}")
        return
    headers = get_auth_header()
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        print()
        _write_output("")
        Print.error(t("commands.shared.invalid_repo_format"))
        print()
        return
    with spinner(t("commands.actions_audit.auditing", repo=repo), color='96'):
        url = f"https://api.github.com/repos/{repo}/contents/.github/workflows"
        resp = requests.get(url, headers=headers)
        if resp.status_code != 200:
            print()
            _write_output("")
            Print.warn(t("commands.actions_audit.no_workflows"))
            return
        workflows = resp.json()
        results = []
        for wf in workflows:
            wf_url = wf.get('download_url')
            name = wf.get('name') or wf.get('path')
            try:
                content = requests.get(wf_url, timeout=8).text
                risky = bool(re.search(r'(curl|wget|bash|sh|powershell|python|node)', content, re.I))
                secrets = bool(re.search(r'secret', content, re.I))
                uses_latest = bool(re.search(r'@latest', content, re.I))
                if risky:
                    status = "risky"
                    message = t("commands.actions_audit.risky_script")
                    note = t("commands.actions_audit.check_exec")
                elif uses_latest:
                    status = "warning"
                    message = t("commands.actions_audit.uses_latest")
                    note = t("commands.actions_audit.pin_versions")
                elif secrets:
                    status = "info"
                    message = t("commands.actions_audit.uses_secrets")
                    note = t("commands.actions_audit.review_secrets")
                else:
                    status = "ok"
                    message = t("commands.actions_audit.ok")
                    note = t("commands.actions_audit.no_risk")
                results.append({
                    'name': name,
                    'url': wf_url,
                    'status': status,
                    'message': message,
                    'note': note
                })
            except Exception:
                results.append({
                    'name': name,
                    'url': wf_url,
                    'status': 'error',
                    'message': t("commands.actions_audit.error_fetch"),
                    'note': t("commands.actions_audit.could_not_fetch")
                })
    print()
    _write_output("")
    _print_audit_results(repo, results)
def _print_audit_results(repo, results):
    """Print actions audit results in a visual, user-friendly format."""
    from ..i18n import t
    safe_print("")
    _write_output("")
    sep = "═" * 60
    safe_print(Print.colorize(sep, '96'))
    _write_output(sep)
    header_text = f"  {t('commands.actions_audit.header')}: {repo}"
    safe_print(Print.colorize(header_text, '97'))
    _write_output(header_text)
    safe_print(Print.colorize(sep, '96'))
    _write_output(sep)
    safe_print("")
    _write_output("")
    total = len(results)
    risky_count = sum(1 for r in results if r['status'] == 'risky')
    warning_count = sum(1 for r in results if r['status'] == 'warning')
    ok_count = sum(1 for r in results if r['status'] == 'ok')
    stats_text = f"  {t('commands.actions_audit.stats', total=total, risky=risky_count)}"
    if risky_count > 0:
        safe_print(Print.colorize(stats_text, '91'))
    elif warning_count > 0:
        safe_print(Print.colorize(stats_text, '93'))
    else:
        safe_print(Print.colorize(stats_text, '92'))
    _write_output(stats_text)
    safe_print("")
    _write_output("")
    for i, result in enumerate(results, 1):
        if result['status'] == 'risky':
            status_label = f"[{t('commands.actions_audit.labels.risky')}]"
            label_color = '91'  
        elif result['status'] == 'warning':
            status_label = f"[{t('commands.actions_audit.labels.warning')}]"
            label_color = '93'  
        elif result['status'] == 'ok':
            status_label = f"[{t('commands.actions_audit.labels.ok')}]"
            label_color = '92'  
        else:
            status_label = f"[{t('commands.actions_audit.labels.error')}]"
            label_color = '90'  
        wf_header = f"  {status_label} {result['name']}"
        safe_print(Print.colorize(wf_header, label_color))
        _write_output(wf_header)
        msg_line = f"         {t('commands.actions_audit.labels.status')}: {result['message']}"
        safe_print(Print.colorize(msg_line, '97'))
        _write_output(msg_line)
        note_line = f"         {t('commands.actions_audit.labels.note')}: {result['note']}"
        if result['status'] == 'risky':
            safe_print(Print.colorize(note_line, '91'))
        elif result['status'] == 'warning':
            safe_print(Print.colorize(note_line, '93'))
        else:
            safe_print(Print.colorize(note_line, '96'))
        _write_output(note_line)
        url_display = result['url']
        if len(url_display) > 55:
            url_display = url_display[:52] + "..."
        url_line = f"         URL: {url_display}"
        safe_print(Print.colorize(url_line, '90'))
        _write_output(url_line)
        if i < len(results):
            safe_print("")
            _write_output("")
    safe_print("")
    _write_output("")
    safe_print(Print.colorize(sep, '96'))
    _write_output(sep)
    safe_print("")
    _write_output("")



import sys
from exc_analyzer.print_utils import Print
def main():
    try:
        from exc_analyzer.cli import main_cli
        main_cli()
    except Exception as e:
        try:
            from exc_analyzer.errors import ExcAnalyzerError
            if isinstance(e, ExcAnalyzerError):
                Print.error(str(e))
            else:
                from exc_analyzer.i18n import t
                Print.error(t("commands.shared.fatal_error", error=e))
        except Exception:
            Print.critical_error(f"Fatal error: {e}")
        sys.exit(1)
if __name__ == "__main__":
    main()



def format_size(bytes_count):
    """Format byte count to human-readable size (KB, MB, GB)."""
    if bytes_count < 1024:
        return f"{bytes_count} bytes"
    elif bytes_count < 1024 * 1024:
        kb = bytes_count / 1024
        return f"{kb:.1f} KB"
    elif bytes_count < 1024 * 1024 * 1024:
        mb = bytes_count / (1024 * 1024)
        return f"{mb:.1f} MB"
    else:
        gb = bytes_count / (1024 * 1024 * 1024)
        return f"{gb:.1f} GB"
def format_friendly_date(date_str, include_relative=False):
    """Format ISO date string to a friendly format (e.g. 22 Sep 2024)."""
    from datetime import datetime, timezone
    from .i18n import get_active_language, t
    if not date_str:
        return ""
    try:
        dt = datetime.fromisoformat(date_str.replace("Z", "+00:00"))
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=timezone.utc)
    except ValueError:
        return date_str
    lang = get_active_language()
    date_text = ""
    if lang == "tr":
        months = [
            "", "Ocak", "Şubat", "Mart", "Nisan", "Mayıs", "Haziran",
            "Temmuz", "Ağustos", "Eylül", "Ekim", "Kasım", "Aralık"
        ]
        date_text = f"{dt.day} {months[dt.month]} {dt.year}"
    else:
        date_text = dt.strftime("%d %b %Y")
    if include_relative:
        rel_text = _get_relative_time(dt)
        return f"{date_text} ({rel_text})"
    return date_text
def _get_relative_time(dt):
    """Calculate relative time string (e.g. 2 years ago)."""
    from datetime import datetime, timezone
    from .i18n import t
    now = datetime.now(timezone.utc)
    diff = now - dt
    seconds = diff.total_seconds()
    if seconds < 0:
        return t("time.just_now") 
    if seconds < 60:
        return t("time.just_now")
    minutes = int(seconds / 60)
    if minutes < 60:
        return f"{minutes} {t('time.minutes' if minutes > 1 else 'time.minute')} {t('time.ago')}"
    hours = int(minutes / 60)
    if hours < 24:
        return f"{hours} {t('time.hours' if hours > 1 else 'time.hour')} {t('time.ago')}"
    days = int(hours / 24)
    if days < 30:
        return f"{days} {t('time.days' if days > 1 else 'time.day')} {t('time.ago')}"
    months = int(days / 30)
    if months < 12:
        return f"{months} {t('time.months' if months > 1 else 'time.month')} {t('time.ago')}"
    years = int(months / 12)
    return f"{years} {t('time.years' if years > 1 else 'time.year')} {t('time.ago')}"
def _truncate(text, width):
    """Truncate text to fit width, adding ellipsis if needed."""
    return truncate_visual(text, width)
def get_visual_width(text):
    """Calculate visual width of string (handling CJK wide chars)."""
    import unicodedata
    w = 0
    for char in str(text):
        cw = unicodedata.east_asian_width(char)
        w += 2 if cw in ('W', 'F') else 1
    return w
def truncate_visual(text, max_width):
    """Truncate string to max_visual_width, adding ... if needed."""
    if not text:
        return ""
    text = str(text)
    current_width = 0
    result = []
    if get_visual_width(text) <= max_width:
        return text
    target_width = max_width - 3 if max_width > 3 else max_width
    for char in text:
        import unicodedata
        cw = unicodedata.east_asian_width(char)
        char_width = 2 if cw in ('W', 'F') else 1
        if current_width + char_width > target_width:
            break
        result.append(char)
        current_width += char_width
    return "".join(result) + "..."
def pad_visual(text, width):
    """Pad string with spaces to reach visual width."""
    v_width = get_visual_width(text)
    padding = width - v_width
    if padding < 0:
        return text 
    return text + (" " * padding)
class TablePrinter:
    """
    Centralized table printing logic with visual width support and dynamic sizing.
    Handles both colored terminal output and clean file output.
    """
    def __init__(self, columns, terminal_width=None):
        """
        Initialize table with column definitions.
        Args:
            columns: List of dicts with keys:
                - header: str (Title)
                - width: int (Fixed width) or None (Flex)
                - color: str (Color code, e.g. '96')
                - align: str ('left', 'right') [default: left]
            terminal_width: Optional manual width override
        """
        import shutil
        self.columns = columns
        self.term_width = terminal_width or shutil.get_terminal_size((100, 20)).columns
        self._calculate_widths()
    def _calculate_widths(self):
        """Calculate actual widths for flex columns."""
        fixed_width = sum(c.get('width', 0) for c in self.columns if c.get('width'))
        gap_size = 2
        total_gaps = (len(self.columns) - 1) * gap_size
        available = self.term_width - fixed_width - total_gaps
        flex_cols = [c for c in self.columns if not c.get('width')]
        if flex_cols:
            flex_width = max(10, available // len(flex_cols))
            for col in flex_cols:
                col['_actual_width'] = flex_width
        for col in self.columns:
            if col.get('width'):
                col['_actual_width'] = col['width']
    def print_header(self):
        """Print the table header."""
        from .print_utils import Print, _write_output, safe_print
        parts_colored = []
        parts_plain = []
        for col in self.columns:
            w = col['_actual_width']
            h = col.get('header', '').upper()
            padded = pad_visual(h, w)
            parts_plain.append(padded)
            parts_colored.append(Print.colorize(padded, '1;97'))
        line_colored = "  ".join(parts_colored)
        line_plain = "   ".join(parts_plain) 
        try:
            print(line_colored)
        except:
            pass
        _write_output(line_plain)
        self.print_separator()
    def print_separator(self):
        """Print dashed separator line."""
        from .print_utils import Print, _write_output, safe_print
        parts = []
        for col in self.columns:
            w = col['_actual_width']
            parts.append("-" * w)
        sep = "  ".join(parts)
        print(Print.colorize(sep, '90'))
        _write_output("   ".join(["-" * col['_actual_width'] for col in self.columns]))
    def print_row(self, data, color_override=None, style_prefix=""):
        """
        Print a single row of data.
        Args:
            data: List of values corresponding to columns.
            color_override: Optional generic color for the whole row (e.g. for dimming).
            style_prefix: Optional ANSI style prefix (e.g. '2;' for dim), prepended to color.
        """
        from .print_utils import Print, _write_output, COLOR_ENABLED
        parts_screen = []
        parts_file = []
        for idx, col in enumerate(self.columns):
            raw_val = data[idx] if idx < len(data) else ""
            w = col['_actual_width']
            url = None
            if isinstance(raw_val, tuple):
                val_text = str(raw_val[0])
                url = str(raw_val[1])
            else:
                val_text = str(raw_val)
            base_color = color_override or col.get('color', '97')
            final_color = f"{style_prefix}{base_color}" if style_prefix else base_color
            truncated = truncate_visual(val_text, w)
            padded = pad_visual(truncated, w)
            colored_text = Print.colorize(padded, final_color)
            if url and COLOR_ENABLED:
                hyperlinked = f"\033]8;;{url}\033\\{colored_text}\033]8;;\033\\"
                parts_screen.append(hyperlinked)
            else:
                parts_screen.append(colored_text)
            file_val = url if url else val_text
            v_width = get_visual_width(file_val)
            padding = w - v_width
            if padding > 0:
                file_padded = file_val + (" " * padding)
            else:
                file_padded = file_val 
            parts_file.append(file_padded)
        try:
            print("  ".join(parts_screen))
        except UnicodeEncodeError:
             print("  ".join(parts_screen).encode('utf-8', errors='replace').decode('utf-8'))
        _write_output("   ".join(parts_file))
def print_bw(label, value, use_white=True):
    from .print_utils import _write_output, safe_print, Print
    color = '97' if use_white else '90'
    label_padded = f"{label:<17}"
    line = f"{Print.colorize(label_padded, color)}: {value}"
    safe_print(line)
def print_bw_list(items, formatter, use_white=True):
    from .print_utils import _write_output, safe_print, Print
    for i, item in enumerate(items):
        color = '97' if (i % 2 == 0) else '90'
        formatted = formatter(item)
        safe_print(Print.colorize(formatted, color))



import sys
import os
import argparse
import difflib
from datetime import datetime
from exc_analyzer.logging_utils import log
from exc_analyzer.print_utils import Print, COLOR_ENABLED, colorize, set_verbose, close_output_file, _write_output, print_error
from exc_analyzer.config import load_key, delete_key
from exc_analyzer.api import notify_new_version, get_version_from_pyproject
from exc_analyzer.i18n import t, set_language, get_active_language, DEFAULT_LANGUAGE
from exc_analyzer.preferences import get_language_preference, set_language_preference
from exc_analyzer.commands.login import cmd_login
from exc_analyzer.commands.logout import cmd_logout
from exc_analyzer.commands.analysis import cmd_analysis
from exc_analyzer.commands.user_a import cmd_user_a
from exc_analyzer.commands.scan_secrets import cmd_scan_secrets
from exc_analyzer.commands.file_history import cmd_file_history
from exc_analyzer.commands.dork_scan import cmd_dork_scan
from exc_analyzer.commands.advanced_secrets import cmd_advanced_secrets
from exc_analyzer.commands.security_score import cmd_security_score
from exc_analyzer.commands.commit_anomaly import cmd_commit_anomaly
from exc_analyzer.commands.user_anomaly import cmd_user_anomaly
from exc_analyzer.commands.content_audit import cmd_content_audit
from exc_analyzer.commands.actions_audit import cmd_actions_audit
class SilentArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        if 'invalid choice:' in message:
            words = message.split()
            attempted = None
            try:
                attempted = message.split('invalid choice:')[1].split('(')[0].strip().strip("'")
            except Exception:
                attempted = None
            commands = {
                'login': ['l', 'log', 'signin', 'sign-in', 'auth'],
                'logout': ['lo', 'log-out', 'signout', 'sign-out', 'exit', 'leave'],
                'user-a': ['u', 'user', 'usera', 'user-audit', 'usr', 'userr', 'usra', 'usr-a'],
                'analysis': ['a', 'ana', 'analys', 'analyzis', 'analiz', 'anlys', 'anyl', 'anali', 'analy'],
                'scan-secrets': ['scan', 'secrets', 'scn', 'scret', 'scrt', 'ss', 's-scan', 'scretscan', 'secscan'],
                'file-history': ['file', 'fileh', 'flhist', 'histfile', 'fh', 'filehist', 'filehis', 'f-history'],
                'dork-scan': ['dork', 'dorkscan', 'drk', 'ds', 'dscan', 'dorks', 'd-sc'],
                'advanced-secrets': ['advsec', 'advsecrets', 'advscrt', 'as', 'adv-s', 'advs', 'advsercet'],
                'security-score': ['secscore', 'sscore', 'sec-score', 'securiscore', 'securityscor', 'ssec', 'securscore'],
                'commit-anomaly': ['commanom', 'commitanom', 'c-anom', 'c-anomaly', 'ca', 'cm-anom', 'comm-anom'],
                'user-anomaly': ['useranom', 'usranom', 'u-anom', 'user-anom', 'ua', 'useranomaly'],
                'content-audit': ['audit', 'contentaudit', 'cntaudit', 'caudit', 'cnt-aud', 'cont-audit'],
                'actions-audit': ['workflow-audit', 'waudit', 'actaudit', 'actionaudit', 'wf-audit', 'wkaudit']
            }
            all_cmds = list(commands.keys()) + [alias for v in commands.values() for alias in v]
            suggestion = None
            if attempted:
                attempted_lower = attempted.lower()
                matches = difflib.get_close_matches(attempted_lower, all_cmds, n=1, cutoff=0.5)
                if matches:
                    for main, aliases in commands.items():
                        if matches[0] == main or matches[0] in aliases:
                            suggestion = main
                            break
            print("")
            Print.error(t("cli.invalid_command"))
            if suggestion:
                print("")
                suggestion_text = t("cli.invalid_command_suggestion", command=suggestion)
                Print.info(suggestion_text)
                print("")
            print("")
        else:
            print("")
            Print.error(message) 
            print("")
        sys.exit(2)
class HelpPrinter:
    """Standardized help message printer."""
    @staticmethod
    def print_help(usage_key, desc_key, examples_key=None, options_key=None, details_key=None, tips_key=None, footer_key=None):
        print("")
        print(colorize(t(usage_key), '96'))
        print("")
        desc = t(desc_key)
        if "https://" in desc:
            words = desc.split()
            colored_words = []
            for w in words:
                if w.startswith("https://"):
                    colored_words.append(colorize(w, '1;94;4'))
                else:
                    colored_words.append(w)
        print(desc)
        if examples_key:
            HelpPrinter._print_section(examples_key, "commands.shared.example_header", "commands.shared.examples_header")
        if options_key:
             HelpPrinter._print_section(options_key, "commands.shared.options_header")
        if details_key:
            HelpPrinter._print_section(details_key, "commands.shared.details_header")
        if tips_key:
            HelpPrinter._print_section(tips_key, "commands.shared.tips_header")
        if footer_key:
            print("")
            print(colorize(t(footer_key), '96'))
        print("")
        sys.exit(0)
    @staticmethod
    def _print_section(data_key, header_single, header_plural=None):
        items = t(data_key)
        if not items:
            return
        if not isinstance(items, list):
            items = [items]
        header = header_plural if (header_plural and len(items) > 1) else header_single
        print("")
        print(colorize(t(header), '93'))
        for item in items:
            print(f"  {item}")
def print_minimal_help():
    cyan = '96' if COLOR_ENABLED else None
    yellow = '93' if COLOR_ENABLED else None
    bold = '1' if COLOR_ENABLED else None
    def c(text, code):
        return colorize(text, code) if code else text
    print(c(r"""
      Y88b   d88P 
       Y88b d88P  
        Y88o88P   
         Y888P         EXC – GitHub Analysis & Security Tool
         d888b                github.com/exc-analyzer
        d88888b   
       d88P Y88b  
      d88P   Y88b 
""", bold))
    cmd_login    = "exc login"
    cmd_analysis = f"exc analysis   {t('cli.min_help.args_repo')}"
    cmd_user     = f"exc user-a     {t('cli.min_help.args_user')}"
    max_len = max(len(cmd_login), len(cmd_analysis), len(cmd_user))
    pad = max_len + 2
    def print_item(cmd, desc):
        prefix = c("[+]", '92')
        print(f"  {prefix} {c(cmd.ljust(pad), cyan)}{c(desc, yellow)}")
    print_item(cmd_login, t("cli.min_help.login_desc"))
    print_item(cmd_analysis, t("cli.min_help.analysis_desc"))
    print_item(cmd_user, t("cli.min_help.user_desc"))
    print("")
    print(c(t("cli.min_help.help_hint"), yellow))
    print(c(t("cli.min_help.detail_hint"), yellow))
    print("")
    sys.exit(0)
def print_full_help():
    cyan = '96' if COLOR_ENABLED else None
    yellow = '93' if COLOR_ENABLED else None
    def c(text, code):
        return colorize(text, code) if code else text
    print("")
    header = f"{c('[+]', '92')} {c(t('cli.full_help.title'), '1')}" 
    print(header)
    print("")
    print(t("cli.full_help.common_header"))
    def arg(k): return t(f"cli.args.{k}")
    common_cmds = [
        (t("cli.full_help.login_logout_usage"), "", "commands.login.desc_combined"),
        ("exc analysis", arg("owner_repo"), "commands.analysis.desc_short"),
        ("exc user-a", arg("username"), "commands.user_a.desc_short"),
        ("exc scan-secrets", arg("owner_repo"), "commands.scan_secrets.desc_short"),
        ("exc file-history", f"{arg('owner_repo')} {arg('file')}", "commands.file_history.desc_short"),
    ]
    sec_cmds = [
        ("exc user-anomaly", arg("username"), "commands.user_anomaly.desc_short"),
        ("exc dork-scan", arg("dork_query"), "commands.dork_scan.desc_short"),
        ("exc advanced-secrets", arg("owner_repo"), "commands.advanced_secrets.desc_short"),
        ("exc security-score", arg("owner_repo"), "commands.security_score.desc_short"),
        ("exc commit-anomaly", arg("owner_repo"), "commands.commit_anomaly.desc_short"),
        ("exc content-audit", arg("owner_repo"), "commands.content_audit.desc_short"),
        ("exc actions-audit", arg("owner_repo"), "commands.actions_audit.desc_short"),
    ]
    gen_opts = [
        ("--version", "(-v)", "cli.general_options.version"),
        ("--verbose", "(-V)", "cli.general_options.verbose"),
        ("--lang", "(-l/-L)", "cli.general_options.lang"),
        ("--show-rate-limit", "(-r)", "cli.general_options.rate_limit"),
    ]
    all_cmds_list = common_cmds + sec_cmds
    max_cmd_len = 0
    for cmd, _, _ in all_cmds_list:
        if len(cmd) > max_cmd_len:
            max_cmd_len = len(cmd)
    cmd_pad = max_cmd_len + 1
    def prepare_cmd_lines(group):
        lines = []
        for cmd, args, desc_key in group:
            prefix = cmd.ljust(cmd_pad)
            if args:
                full_usage = f"{prefix} {args}"
            else:
                full_usage = cmd 
            lines.append((full_usage, desc_key))
        return lines
    common_lines = prepare_cmd_lines(common_cmds)
    sec_lines = prepare_cmd_lines(sec_cmds)
    max_len_cmds = 0
    for usage, _ in common_lines + sec_lines:
        if len(usage) > max_len_cmds:
            max_len_cmds = len(usage)
    desc_pad_cmds = max_len_cmds + 4
    max_long = 0
    for long_f, _, _ in gen_opts:
        if len(long_f) > max_long:
            max_long = len(long_f)
    long_pad = max_long + 4  
    max_short = 0
    for _, short_f, _ in gen_opts:
        if len(short_f) > max_short:
            max_short = len(short_f)
    short_pad = max_short + 4 
    def p_lines(lines, pad, color_code):
        for usage, key in lines:
            left = f"  {usage}".ljust(pad)
            desc = t(key)
            print(c(left, cyan) + c(desc, color_code))
    p_lines(common_lines, desc_pad_cmds, yellow)
    print("")
    print(t("cli.full_help.security_header"))
    p_lines(sec_lines, desc_pad_cmds, yellow)
    print("")
    print(t("cli.full_help.general_options_header"))
    for long_f, short_f, key in gen_opts:
        long_part = f"  {long_f}".ljust(long_pad + 2) 
        short_part = f"{short_f}".ljust(short_pad)
        left = f"{long_part}{short_part}"
        desc = f"# {t(key)}"
        print(c(left, cyan) + c(desc, yellow))
    print("")
    print("")
    print(t("cli.full_help.info_hint"))
    print("")
    notify_new_version()
    print("")
    sys.exit(0)
def _extract_cli_language(argv):
    lang = None
    for idx, arg in enumerate(argv):
        if arg in ("--lang", "-L", "-l"):
            if idx + 1 < len(argv):
                potential_lang = argv[idx + 1]
                if arg == "-l" and potential_lang.isdigit():
                    continue
                lang = potential_lang
            break
        if arg.startswith("--lang="):
            lang = arg.split("=", 1)[1]
            break
    return lang
def configure_language():
    """Resolve the active language from CLI flag, env vars, or saved settings."""
    cli_lang = _extract_cli_language(sys.argv)
    env_lang = os.environ.get("EXC_LANG") or os.environ.get("LANG")
    saved_lang = get_language_preference()
    if cli_lang:
        if set_language(cli_lang):
            active = get_active_language()
            set_language_preference(active)
            return active
        Print.warn(t("cli.language_not_available", lang=cli_lang, fallback=DEFAULT_LANGUAGE))
        set_language(DEFAULT_LANGUAGE)
        return DEFAULT_LANGUAGE
    if env_lang and set_language(env_lang):
        return get_active_language()
    if saved_lang:
        if set_language(saved_lang):
            return get_active_language()
    set_language(DEFAULT_LANGUAGE)
    return DEFAULT_LANGUAGE
def main_cli():
    configure_language()
    notify_new_version()
    if "--version" in sys.argv or "-v" in sys.argv:
        notify_new_version()
        print("")
        local_version = get_version_from_pyproject() or t("cli.version_missing")
        print(f"EXC Analyzer v{local_version}")
        print("")
        sys.exit(0)
        print("")
        sys.exit(0) 
    if len(sys.argv) == 1 or (len(sys.argv) > 1 and sys.argv[1] == "exc"):
        print_minimal_help() 
        sys.exit(0)
    if sys.argv[1] in ("-h", "--help", "help"):
        print_full_help()  
        sys.exit(0)
    if "--verbose" in sys.argv or "-V" in sys.argv:
        set_verbose(True)
        Print.warn(t("cli.verbose_enabled"))
        sys.argv = [a for a in sys.argv if a not in ["--verbose", "-V"]]
    parser = SilentArgumentParser(
        prog="exc",
        usage="",
        description="",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    parser.add_argument(
        "-L", "-l",
        "--lang",
        metavar=t("cli.lang_option_metavar"),
        help=t("cli.general_options.lang")
    )
    subparsers = parser.add_subparsers(dest="command")
    login_parser = subparsers.add_parser(
        "login",
        description="Authenticate via GitHub Device Flow.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    login_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def login_help(args):
        print("")
        print(colorize(t("commands.login.usage"), '96'))
        print("")
        desc = t("commands.login.description")
        url = "https://github.com/login/device"
        desc = desc.replace(url, colorize(url, '1;94;4'))
        parts = desc.split('\n', 1)
        if len(parts) > 0:
            parts[0] = colorize(parts[0], '1;97')
            desc = '\n'.join(parts)
        print(desc)
        sys.exit(0)
    login_parser.set_defaults(func=cmd_login, help_func=login_help)
    logout_parser = subparsers.add_parser(
        "logout",
        description="Log out and remove API key.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    logout_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def logout_help(args):
        print("")
        print(colorize(t("commands.logout.usage"), '96'))
        print("")
        desc = t("commands.logout.description")
        cmd_str = '"exc login"'
        desc = desc.replace(cmd_str, colorize(cmd_str, '93'))
        print(desc)
        sys.exit(0)
    logout_parser.set_defaults(func=cmd_logout, help_func=logout_help)
    analysis_parser = subparsers.add_parser(
        "analysis",
        description="Repository analysis: code, security, dependencies, stats.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    analysis_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    analysis_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    analysis_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    analysis_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def analysis_help(args):
        HelpPrinter.print_help(
            "commands.analysis.usage",
            "commands.analysis.description",
            examples_key="commands.analysis.examples",
            options_key="commands.analysis.options"
        )
    analysis_parser.set_defaults(func=cmd_analysis, help_func=analysis_help)
    user_parser = subparsers.add_parser(
        "user-a",
        description="Analyze a GitHub user's profile and repositories.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    user_parser.add_argument("username", nargs="?", help=argparse.SUPPRESS)
    user_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    user_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    user_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def user_help(args):
        HelpPrinter.print_help(
            "commands.user_a.usage",
            "commands.user_a.description",
            examples_key="commands.user_a.examples",
            details_key="commands.user_a.details",
            tips_key="commands.user_a.tip" 
        )
    user_parser.set_defaults(func=cmd_user_a, help_func=user_help)
    scan_parser = subparsers.add_parser(
        "scan-secrets",
        description="Scan recent commits for leaked secrets.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    scan_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    scan_parser.add_argument("-l", "--limit", type=int, default=10, help=t("commands.shared.help_limit", default=10))
    scan_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    scan_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    scan_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def scan_help(args):
        HelpPrinter.print_help(
            "commands.scan_secrets.usage",
            "commands.scan_secrets.description",
            examples_key="commands.scan_secrets.examples",
            options_key="commands.scan_secrets.options"
        )
    scan_parser.set_defaults(func=cmd_scan_secrets, help_func=scan_help)
    file_parser = subparsers.add_parser(
        "file-history",
        description="Show the change history of a file in a repository.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    file_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    file_parser.add_argument("filepath", nargs="?", help=argparse.SUPPRESS)
    file_parser.add_argument("-l", "--limit", type=int, default=5, help=t("commands.shared.help_limit", default=5))
    file_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    file_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    file_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def file_help(args):
        HelpPrinter.print_help(
            "commands.file_history.usage",
            "commands.file_history.description",
            examples_key="commands.file_history.examples",
            options_key="commands.file_history.options"
        )
    file_parser.set_defaults(func=cmd_file_history, help_func=file_help)
    dork_parser = subparsers.add_parser(
        "dork-scan",
        description="Scan GitHub for sensitive keywords or patterns (dorking).",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    dork_parser.add_argument("query", nargs="*", help=argparse.SUPPRESS)
    dork_parser.add_argument("-n", "--num", type=int, default=10, help=t("commands.shared.help_num"))
    dork_parser.add_argument("-p", "--preset", help=t("commands.dork_scan.help_preset")) 
    dork_parser.add_argument("--verify", action="store_true", help=t("commands.dork_scan.help_verify"))
    dork_parser.add_argument("--list-presets", action="store_true", help=t("commands.dork_scan.help_list_presets"))
    dork_parser.add_argument("--export", metavar="FILE", help=t("commands.dork_scan.help_export"))
    dork_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    dork_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    dork_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def dork_help(args):
        HelpPrinter.print_help(
            "commands.dork_scan.usage",
            "commands.dork_scan.description",
            examples_key="commands.dork_scan.examples",
            options_key="commands.dork_scan.options",
            details_key="commands.dork_scan.details",
            tips_key="commands.dork_scan.tips",
            footer_key="commands.dork_scan.footer"
        )
    dork_parser.set_defaults(func=cmd_dork_scan, help_func=dork_help)
    advsec_parser = subparsers.add_parser(
        "advanced-secrets",
        description="Scan repo for a wide range of secret patterns.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    advsec_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    advsec_parser.add_argument("-l", "--limit", type=int, default=20, help=t("commands.shared.help_limit", default=20))
    advsec_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    advsec_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    advsec_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def advsec_help(args):
        HelpPrinter.print_help(
            "commands.advanced_secrets.usage",
            "commands.advanced_secrets.description",
            examples_key="commands.advanced_secrets.examples",
            options_key="commands.advanced_secrets.options"
        )
    advsec_parser.set_defaults(func=cmd_advanced_secrets, help_func=advsec_help)
    secscore_parser = subparsers.add_parser(
        "security-score",
        description="Calculate a security score for the repository.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    secscore_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    secscore_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    secscore_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    secscore_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def secscore_help(args):
        HelpPrinter.print_help(
            "commands.security_score.usage",
            "commands.security_score.description",
            examples_key="commands.security_score.examples"
        )
    secscore_parser.set_defaults(func=cmd_security_score, help_func=secscore_help)
    commanom_parser = subparsers.add_parser(
        "commit-anomaly",
        description="Analyze commit/PR activity for anomalies.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    commanom_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    commanom_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    commanom_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    commanom_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def commanom_help(args):
        HelpPrinter.print_help(
            "commands.commit_anomaly.usage",
            "commands.commit_anomaly.description",
            examples_key="commands.commit_anomaly.examples"
        )
    commanom_parser.set_defaults(func=cmd_commit_anomaly, help_func=commanom_help)
    useranom_parser = subparsers.add_parser(
        "user-anomaly",
        description="Detect unusual activity in a user's GitHub activity.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    useranom_parser.add_argument("username", nargs="?", help=argparse.SUPPRESS)
    useranom_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    useranom_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    useranom_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def useranom_help(args):
        HelpPrinter.print_help(
            "commands.user_anomaly.usage",
            "commands.user_anomaly.description",
            examples_key="commands.user_anomaly.examples"
        )
    useranom_parser.set_defaults(func=cmd_user_anomaly, help_func=useranom_help)
    content_parser = subparsers.add_parser(
        "content-audit",
        description="Audit repo for license, security.md, docs, etc.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    content_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    content_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    content_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    content_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def content_help(args):
        HelpPrinter.print_help(
            "commands.content_audit.usage",
            "commands.content_audit.description",
            examples_key="commands.content_audit.examples"
        )
    content_parser.set_defaults(func=cmd_content_audit, help_func=content_help)
    actions_parser = subparsers.add_parser(
        "actions-audit",
        description="Audit GitHub Actions/CI workflows for security.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        add_help=False
    )
    actions_parser.add_argument("repo", nargs="?", help=argparse.SUPPRESS)
    actions_parser.add_argument("-o", "--output", nargs='?', const='', help=t("commands.shared.help_output"))
    actions_parser.add_argument("-r", "--show-rate-limit", action="store_true", help=argparse.SUPPRESS)
    actions_parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    def actions_help(args):
        HelpPrinter.print_help(
            "commands.actions_audit.usage",
            "commands.actions_audit.description",
            examples_key="commands.actions_audit.examples"
        )
    actions_parser.set_defaults(func=cmd_actions_audit, help_func=actions_help)
    parser.add_argument("-h", "--help", action="store_true", help=argparse.SUPPRESS)
    if len(sys.argv) > 1:
        sys.argv[1] = sys.argv[1].lower()
    args, unknown = parser.parse_known_args()
    if unknown:
        print("")
        Print.error(t("cli.unknown_args", args=' '.join(unknown)))
        print("")
        sys.exit(1)
    if hasattr(args, 'help') and args.help:
        if hasattr(args, 'help_func'):
            args.help_func(args)
        else:
            print_full_help()
    if args.command == "dork-scan" and not args.query and not getattr(args, 'preset', None) and not getattr(args, 'list_presets', False):
        print_error(t("commands.dork_scan.missing_query"))
        sys.exit(1)
    if hasattr(args, 'func'):
        try:
            output_file = getattr(args, 'output', None)
            if output_file is not None:
                if not output_file or output_file == '':
                    timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
                    cmd = getattr(args, 'command', 'output')
                    filename_parts = [cmd]
                    if hasattr(args, 'repo') and args.repo:
                        repo_name = args.repo.replace('/', '_')
                        filename_parts.append(repo_name)
                    elif hasattr(args, 'username') and args.username:
                        filename_parts.append(args.username)
                    elif hasattr(args, 'query') and args.query:
                        if isinstance(args.query, list):
                            query_str = '_'.join(args.query)[:20]
                        else:
                            query_str = str(args.query)[:20]
                        query_safe = query_str.replace(' ', '_').replace('/', '_')
                        filename_parts.append(query_safe)
                    filename_parts.append(timestamp)
                    output_file = f"{'_'.join(filename_parts)}.txt"
                from exc_analyzer.print_utils import set_output_file
                set_output_file(output_file)
            args.func(args)
            if output_file:
                close_output_file()
        except KeyboardInterrupt:
            print("")
            from exc_analyzer.print_utils import print_cancelled
            print_cancelled(t("commands.shared.scan_cancelled"))
            try:
                close_output_file()
            except Exception:
                pass
            sys.exit(130)  
        except Exception as e:
            try:
                from exc_analyzer.errors import ExcAnalyzerError
                if isinstance(e, ExcAnalyzerError):
                    error_msg = str(e)
                else:
                    error_msg = t("commands.shared.error_occurred", error=str(e))
                print_error(error_msg)
            except Exception as ex:
                try:
                    from exc_analyzer.print_utils import print_critical_error
                    print_critical_error(str(e))
                except:
                    print(f"FATAL ERROR: {e}")
            try:
                close_output_file()
            except Exception:
                pass
            from exc_analyzer.logging_utils import log
            log(f"Command error: {e}")
            sys.exit(1)
    else:
        print_full_help()



#!/usr/bin/env python3
import os
import base64
from exc_analyzer.constants import CONFIG_DIR
from exc_analyzer.i18n import t
KEY_FILE = os.path.join(CONFIG_DIR, "build.sec")
try:
    import keyring
    KEYRING_AVAILABLE = True
except Exception:
    KEYRING_AVAILABLE = False
def ensure_config_dir():
    if not os.path.exists(CONFIG_DIR):
        try:
            os.makedirs(CONFIG_DIR, mode=0o700, exist_ok=True)
            if os.name == 'nt':
                try:
                    import subprocess
                    subprocess.run(['icacls', CONFIG_DIR, '/inheritance:r', '/grant:r', '%username%:(OI)(CI)F'], 
                                   capture_output=True, check=False)
                except Exception:
                    pass
            else:
                try:
                    os.chmod(CONFIG_DIR, 0o700)
                except Exception:
                    pass
        except Exception:
            os.makedirs(CONFIG_DIR, exist_ok=True)
def save_key(key: str, silent: bool = False):
    from exc_analyzer.print_utils import Print
    ensure_config_dir()
    if KEYRING_AVAILABLE:
        try:
            keyring.set_password("exc-analyzer", "github_token", key)
            if not silent:
                Print.info(t("commands.shared.config_saved_keyring"))
                from exc_analyzer.print_utils import safe_print
                safe_print("")
            return
        except Exception:
            pass
    encoded = base64.b64encode(key.encode('utf-8')).decode('utf-8')
    try:
        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC
        fd = os.open(KEY_FILE, flags, 0o600)
        with os.fdopen(fd, 'w', encoding='utf-8') as f:
            f.write(encoded)
    except Exception:
        with open(KEY_FILE, "w", encoding="utf-8") as f:
            f.write(encoded)
        try:
            os.chmod(KEY_FILE, 0o600)
        except Exception:
            Print.warn(t("commands.shared.config_perm_warning"))
    if not silent:
        Print.info(t("commands.shared.config_saved_local"))
        from exc_analyzer.print_utils import safe_print
        safe_print("")
def load_key():
    if KEYRING_AVAILABLE:
        try:
            val = keyring.get_password("exc-analyzer", "github_token")
            if val:
                return val
        except Exception:
            pass
    if not os.path.isfile(KEY_FILE):
        return None
    try:
        with open(KEY_FILE, "r", encoding="utf-8") as f:
            encoded = f.read()
            key = base64.b64decode(encoded).decode('utf-8')
            return key
    except Exception:
        return None
def delete_key():
    removed = False
    if KEYRING_AVAILABLE:
        try:
            existing = keyring.get_password("exc-analyzer", "github_token")
            if existing:
                keyring.delete_password("exc-analyzer", "github_token")
                removed = True
        except Exception:
            pass
    if os.path.isfile(KEY_FILE):
        try:
            os.remove(KEY_FILE)
            removed = True
        except Exception:
            try:
                with open(KEY_FILE, 'w', encoding='utf-8') as f:
                    f.write('')
                os.remove(KEY_FILE)
                removed = True
            except Exception:
                pass
    return removed
def validate_key(key):
    pass



from exc_analyzer.config import delete_key
from exc_analyzer.i18n import t
from exc_analyzer.print_utils import Print, colorize
def cmd_logout(args):
    """
    Handle the logout process (delete stored API key).
    """
    from exc_analyzer.config import load_key
    if not load_key():
        from exc_analyzer.print_utils import print_warning, safe_print, _write_output
        safe_print("")
        _write_output("")
        print_warning(t("commands.logout.not_logged_in"))
        safe_print("")
        _write_output("")
        return
    from exc_analyzer.print_utils import print_info, print_success, safe_print, _write_output
    safe_print("")
    _write_output("")
    delete_key()
    print_success(t("commands.logout.success"))
    print_info(t("commands.logout.hint"))
    safe_print("")
    _write_output("")



import requests
from ..print_utils import Print
from ..api import get_auth_header, get_all_pages
from ..i18n import t
def cmd_contrib_impact(args):
    """Measure contributor impact using line changes"""
    from ..print_utils import _write_output, safe_print
    headers = get_auth_header()
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        safe_print("")
        Print.error(t("commands.shared.invalid_repo_format"))
        safe_print("")
        return
    stats_url = f"https://api.github.com/repos/{repo}/stats/contributors"
    contributors = get_all_pages(stats_url, headers)
    safe_print("")
    header_msg = t("commands.contrib_impact.header", repo=repo)
    Print.info(header_msg.strip())
    formula_msg = t("commands.contrib_impact.formula")
    safe_print(Print.colorize(f"  {formula_msg}", '90')) 
    _write_output(formula_msg)
    results = []
    if contributors:
        for contributor in contributors:
            if not contributor or not contributor.get('author'):
                continue
            login = contributor['author']['login']
            total_add = sum(w['a'] for w in contributor['weeks'])
            total_del = sum(w['d'] for w in contributor['weeks'])
            score = (total_add * 0.7) - (total_del * 0.3)
            results.append((login, score, total_add, total_del))
    if not results:
        safe_print("")
        Print.warn(t("commands.shared.none_found")) 
        safe_print("")
        return
    safe_print("")
    Print.success(t("commands.contrib_impact.top_header").strip())
    for login, score, adds, dels in sorted(results, key=lambda x: x[1], reverse=True)[:10]:
        safe_print("")
        user_line = f"  {login}"
        safe_print(Print.colorize(user_line, '1;97')) 
        _write_output(user_line)
        score_val = f"{score:.1f}"
        score_txt = t("commands.contrib_impact.score_display", score=score_val)
        Print.info(score_txt)
        stats_txt = t("commands.contrib_impact.stats_display", adds=adds, dels=dels)
        safe_print(f"      {stats_txt}")
        _write_output(f"      {stats_txt}")
    safe_print("")



import json
from functools import lru_cache
from typing import Any, Dict, Iterable, Optional
from pathlib import Path
DEFAULT_LANGUAGE = "en"
_py_path = Path(__file__).resolve().parent
if _py_path.name == "__pycache__":
    _py_path = _py_path.parent
_LOCALE_ROOT = _py_path / "locale"
_active_language = DEFAULT_LANGUAGE
def _normalize(code: Optional[str]) -> str:
    if not code:
        return DEFAULT_LANGUAGE
    token = code.strip().lower()
    if "." in token:
        token = token.split(".", 1)[0]
    if "_" in token:
        token = token.split("_", 1)[0]
    return token or DEFAULT_LANGUAGE
@lru_cache()
def available_languages() -> Iterable[str]:
    languages = set()
    try:
        for entry in _LOCALE_ROOT.iterdir():
            if not entry.is_dir():
                continue
            catalog = entry / "messages.json"
            if catalog.is_file():
                languages.add(entry.name.lower())
    except FileNotFoundError:
        return tuple([DEFAULT_LANGUAGE])
    languages.add(DEFAULT_LANGUAGE)
    return tuple(sorted(languages))
@lru_cache()
def _load_catalog(language: str) -> Dict[str, Any]:
    normalized = _normalize(language)
    catalog_path = _LOCALE_ROOT / normalized / "messages.json"
    try:
        with catalog_path.open("r", encoding="utf-8") as handle:
            return json.load(handle)
    except (FileNotFoundError, json.JSONDecodeError):
        return {}
def _resolve_key(catalog: Dict[str, Any], key: str) -> Any:
    current: Any = catalog
    for part in key.split('.'):
        if isinstance(current, dict) and part in current:
            current = current[part]
        else:
            return None
    return current
def set_language(language: Optional[str]) -> bool:
    """Set the active language if available; returns True on success."""
    global _active_language
    normalized = _normalize(language)
    catalog_path = _LOCALE_ROOT / normalized / "messages.json"
    if normalized not in available_languages() and not catalog_path.is_file():
        _active_language = DEFAULT_LANGUAGE
        return False
    _active_language = normalized
    return True
def get_active_language() -> str:
    return _active_language
def reset_language_cache() -> None:
    _load_catalog.cache_clear()
    available_languages.cache_clear()
def _lookup(key: str, language: str) -> Any:
    catalog = _load_catalog(language)
    if not catalog:
        return None
    return _resolve_key(catalog, key)
def t(key: str, **kwargs: Any) -> Any:
    """Translate the dotted key using the active language with fallback."""
    value = _lookup(key, _active_language)
    if value is None and _active_language != DEFAULT_LANGUAGE:
        value = _lookup(key, DEFAULT_LANGUAGE)
    if value is None:
        value = key
    if isinstance(value, str) and kwargs:
        try:
            return value.format(**kwargs)
        except KeyError:
            return value
    return value



import json
import os
from typing import Any, Dict
from exc_analyzer.constants import CONFIG_DIR
from exc_analyzer.config import ensure_config_dir
SETTINGS_FILE = os.path.join(CONFIG_DIR, "settings.json")
DEFAULT_SETTINGS: Dict[str, Any] = {
    "language": "en"
}
def _read_settings_file() -> Dict[str, Any]:
    ensure_config_dir()
    if not os.path.isfile(SETTINGS_FILE):
        return DEFAULT_SETTINGS.copy()
    try:
        with open(SETTINGS_FILE, "r", encoding="utf-8") as handle:
            data = json.load(handle)
            if isinstance(data, dict):
                merged = DEFAULT_SETTINGS.copy()
                merged.update(data)
                return merged
    except (json.JSONDecodeError, OSError):
        pass
    return DEFAULT_SETTINGS.copy()
def _write_settings_file(settings: Dict[str, Any]) -> None:
    ensure_config_dir()
    payload = DEFAULT_SETTINGS.copy()
    payload.update(settings)
    with open(SETTINGS_FILE, "w", encoding="utf-8") as handle:
        json.dump(payload, handle, indent=2, sort_keys=True)
def load_settings() -> Dict[str, Any]:
    """Return a copy of persisted CLI settings."""
    return _read_settings_file()
def save_settings(settings: Dict[str, Any]) -> None:
    """Persist the provided settings dictionary."""
    _write_settings_file(settings)
def get_language_preference() -> str:
    """Fetch the saved language preference (defaults to English)."""
    return str(load_settings().get("language", DEFAULT_SETTINGS["language"]))
def set_language_preference(language: str) -> None:
    """Persist the selected language preference."""
    settings = load_settings()
    settings["language"] = language
    save_settings(settings)



"""Loading spinner for CLI operations."""
import sys
import threading
import time
from .print_utils import COLOR_ENABLED
class Spinner:
    """Thread-based loading spinner for CLI."""
    FRAMES = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏']
    @staticmethod
    def _is_powershell():
        """Check if running in PowerShell."""
        import os
        return os.environ.get('PSModulePath') is not None or \
               'powershell' in os.environ.get('PROMPT', '').lower() or \
               'pwsh' in os.environ.get('PROMPT', '').lower()
    def __init__(self, message="Loading", color='96'):
        """
        Initialize spinner.
        Args:
            message: Text to display alongside spinner
            color: ANSI color code (default: cyan 96)
        """
        self.message = message
        self.color = color
        self._stop_event = threading.Event()
        self._thread = None
        self._frame_index = 0
        self._is_powershell = self._is_powershell()
    def _spin(self):
        """Internal spinner loop (runs in thread)."""
        import shutil
        last_len = 0
        try:
            while not self._stop_event.is_set():
                frame = self.FRAMES[self._frame_index % len(self.FRAMES)]
                cols = shutil.get_terminal_size((80, 20)).columns
                max_msg_len = max(5, cols - 5) 
                msg = self.message
                current_len = len(f"{frame} {msg}")
                if current_len > max_msg_len:
                    available = max_msg_len - len(f"{frame} ...")
                    if available > 0:
                        msg = msg[:available] + "..."
                    else:
                        msg = "..."
                if COLOR_ENABLED:
                    output = f"\r\033[K\033[{self.color}m{frame}\033[0m {msg}"
                else:
                    output = f"\r{frame} {msg}"
                    prev_len = last_len
                    curr_len = len(output)
                    if curr_len < prev_len:
                         output += " " * (prev_len - curr_len)
                sys.stdout.write(output)
                sys.stdout.flush()
                if COLOR_ENABLED:
                    last_len = len(f"{frame} {msg}") + 5 
                else:
                    last_len = len(output)
                self._frame_index += 1
                time.sleep(0.1)
        except Exception:
            pass
        finally:
            try:
                if COLOR_ENABLED:
                    sys.stdout.write('\r\033[K')
                else:
                    sys.stdout.write('\r' + ' ' * max(last_len, 40) + '\r')
                sys.stdout.flush()
            except Exception:
                pass
    def start(self):
        """Start the spinner animation."""
        if self._thread is not None:
            return  
        self._stop_event.clear()
        self._thread = threading.Thread(target=self._spin, daemon=True)
        self._thread.start()
    def stop(self, final_message=None):
        """
        Stop the spinner animation.
        Args:
            final_message: Optional message to display after stopping
        """
        if self._thread is None:
            return  
        self._stop_event.set()
        self._thread.join(timeout=0.5)
        self._thread = None
        sys.stdout.write('\r')
        if COLOR_ENABLED:
             sys.stdout.write('\033[K')
        sys.stdout.flush()
        if final_message:
            print(final_message)
    def update_message(self, new_message):
        """Update spinner message while running."""
        self.message = new_message
    def __enter__(self):
        """Context manager entry."""
        self.start()
        return self
    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit."""
        self.stop()
        return False
def spinner(message="Loading", color='96'):
    """
    Create a spinner context manager.
    Usage:
        with spinner("Fetching data"):
            # Long running operation
            time.sleep(5)
    Args:
        message: Text to display
        color: ANSI color code
    Returns:
        Spinner instance
    """
    return Spinner(message, color)



"""Advanced secret scanning with concurrent file processing."""
import asyncio
import re
from ..print_utils import Print
from ..async_api import create_async_context
from ..config import load_key
SECRET_PATTERNS = {
    'AWS Key': r'AKIA[0-9A-Z]{16}',
    'GitHub Token': r'ghp_[a-zA-Z0-9]{36}',
    'Slack Token': r'xox[baprs]-([0-9a-zA-Z]{10,48})?',
    'Google API Key': r'AIza[0-9A-Za-z\-_]{35}',
    'Heroku API Key': r'heroku[a-z0-9]{32}',
    'Discord Token': r'[MN][A-Za-z\d]{23}\.[\\w-]{6}\.[\\w-]{27}',
    'Stripe Key': r'sk_live_[0-9a-zA-Z]{24}',
    'Mailgun Key': r'key-[0-9a-zA-Z]{32}',
    'API Key': r'(?i)(api|access)[_ -]?key["\']?[:=] ?["\'][0-9a-zA-Z]{20,40}',
    'Config File': r'(config|settings|secret|credentials|env)(\.json|\.yml|\.yaml|\.py|\.env)'
}
from ..i18n import t
async def _scan_advanced_secrets_async(repo: str, token: str, commit_limit: int = 20, show_rate_limit: bool = False):
    """Advanced secret scanning with concurrent file processing."""
    from ..print_utils import _write_output, safe_print
    from ..spinner import spinner
    async with create_async_context(token) as client:
        Print.info(t("commands.advanced_secrets.scanning_repo", limit=commit_limit, repo=repo))
        safe_print("")
        _write_output("")
        found = []
        try:
            with spinner(t("commands.advanced_secrets.scanning_files"), color='96'):
                tree_url = f"https://api.github.com/repos/{repo}/git/trees/HEAD?recursive=1"
                tree_data = await client.fetch_json(tree_url)
                tree_items = tree_data.get('tree', [])
            suspect_files = []
            for item in tree_items:
                if item['type'] == 'blob':
                    path = item.get('path', '')
                    if any(ext in path.lower() for ext in ['.env', 'config', 'secret', 'credential', '.json', '.yml', '.yaml', '.py']):
                        suspect_files.append(item)
            sem = asyncio.Semaphore(10)
            file_tasks = []
            async def _scan_with_sem(url, path, source):
                async with sem:
                    return await _scan_file(url, path, source)
            for file_item in suspect_files[:50]:  
                raw_url = f"https://raw.githubusercontent.com/{repo}/HEAD/{file_item['path']}"
                task = asyncio.create_task(_scan_with_sem(raw_url, file_item['path'], 'file'))
                file_tasks.append(task)
            file_results = await asyncio.gather(*file_tasks, return_exceptions=True)
            found.extend([item for r in file_results if isinstance(r, list) for item in r])
            Print.info(t("commands.advanced_secrets.scanning_commits"))
            commits_url = f"https://api.github.com/repos/{repo}/commits"
            commits = await client.fetch_paginated(commits_url, per_page=commit_limit, max_pages=1)
            commit_tasks = []
            for commit in commits[:commit_limit]:
                task = asyncio.create_task(_scan_commit(client, commit, repo))
                commit_tasks.append(task)
            commit_results = await asyncio.gather(*commit_tasks, return_exceptions=True)
            found.extend([item for result in commit_results if result for item in result])
            safe_print("")
            _write_output("")
            if found:
                Print.info(t("commands.advanced_secrets.total_findings", count=len(found)))
                print()
                _write_output(f"\n{t('commands.advanced_secrets.total_findings', count=len(found))}\n")
                for path, secret_type, url, source in found:
                    file_part = t("commands.advanced_secrets.output.file", path=path)
                    type_part = t("commands.advanced_secrets.output.type", type=secret_type)
                    header = f"[+] {path} "
                    safe_print(Print.colorize(header, '94'), end="")
                    safe_print(Print.colorize(f"({secret_type})", '93'))
                    _write_output(f"[+] {path} ({secret_type})")
                    source_line = f"    {t('commands.advanced_secrets.output.source')}: {source}"
                    safe_print(Print.colorize(source_line, '97'))
                    _write_output(source_line)
                    display_url = url
                    if "github.com" in url and len(url) > 60:
                        for marker in ["/raw/", "/blob/", "/commit/"]:
                            if marker in url:
                                parts = url.split(marker)
                                if len(parts) == 2:
                                    base = parts[0]
                                    rest = parts[1]
                                    if len(rest) > 20:
                                        display_url = f"{base}{marker}.../{rest.split('/')[-1]}"
                                        break
                        if display_url == url: 
                             display_url = url[:50] + "..." + url[-10:]
                    indent = "    "
                    safe_print(indent, end="") 
                    Print.hyperlink(display_url, url)
                    _write_output(f"    {url}")
                    print()
                    _write_output("")
            else:
                Print.success(t("commands.advanced_secrets.none_found"))
            safe_print("")
            _write_output("")
            if show_rate_limit:
                Print.info(client.get_quota_info())
                safe_print("")
                _write_output("")
        except Exception as e:
            Print.error(f"Scan failed: {str(e)}")
async def _scan_file(url: str, path: str, source: str) -> list:
    """Scan a single file for secrets."""
    results = []
    try:
        import aiohttp
        async with aiohttp.ClientSession() as session:
            async with session.get(url, timeout=aiohttp.ClientTimeout(total=10)) as resp:
                if resp.status == 200:
                    content = await resp.text()
                    for secret_type, pattern in SECRET_PATTERNS.items():
                        if re.search(pattern, content):
                            results.append([path, secret_type, url, source])
                            break
    except Exception as e:
        from ..logging_utils import log
        log(f"Failed to scan file {url}: {e}")
    return results
async def _scan_commit(client, commit: dict, repo: str) -> list:
    """Scan a commit for secrets."""
    results = []
    try:
        commit_url = commit.get('url')
        if not commit_url:
            return results
        commit_data = await client.fetch_json(commit_url)
        files = commit_data.get('files', [])
        sem = asyncio.Semaphore(10)
        file_tasks = []
        async def _scan_with_sem(url, path, source):
            async with sem:
                return await _scan_file(url, path, source)
        for file in files:
            if file.get('status') in ['added', 'modified']:
                raw_url = file.get('raw_url')
                if raw_url:
                    source = f"commit: {commit.get('sha', '')[:7]}"
                    task = asyncio.create_task(_scan_with_sem(raw_url, file.get('filename'), source))
                    file_tasks.append(task)
        file_results = await asyncio.gather(*file_tasks, return_exceptions=True)
        for result in file_results:
            if isinstance(result, list):
                results.extend(result)
        return results
    except Exception:
        return results
def cmd_advanced_secrets(args):
    """Advanced secret scanning - optimized with concurrent processing."""
    from ..print_utils import _write_output
    if not args.repo:
        Print.error(t("commands.shared.missing_repo"))
        _write_output(f"\n{t('commands.advanced_secrets.usage')}")
        return
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        print("")
        _write_output("")
        Print.error(t("commands.shared.invalid_repo_format"))
        print("")
        return
    commit_limit = getattr(args, 'limit', 20)
    token = load_key()
    if not token:
        print("")
        _write_output("")
        Print.error(t("commands.shared.api_key_missing"))
        Print.info(t("commands.shared.use_key_cmd"))
        print("")
        _write_output("")
        return
    try:
        print("")
        _write_output("")
        show_rate = getattr(args, 'show_rate_limit', False)
        asyncio.run(_scan_advanced_secrets_async(repo, token, commit_limit, show_rate))
    except KeyboardInterrupt:
        print("")
        Print.warn(t("commands.shared.scan_cancelled"))
    except Exception as e:
        Print.error(t("commands.shared.error_occurred", error=str(e)))



import requests
from ..print_utils import Print, _write_output, safe_print
from ..api import api_get, get_auth_header, get_all_pages
from ..spinner import spinner
from ..i18n import t
from ..helpers import format_friendly_date
def _search_file_in_repo(repo, filename, headers):
    """
    Search for a file in the repository using GitHub Code Search API.
    Returns the full path if found, otherwise None.
    """
    search_url = f"https://api.github.com/search/code?q=filename:{filename}+repo:{repo}"
    try:
        result, _ = api_get(search_url, headers)
        if result and isinstance(result, dict) and result.get('total_count', 0) > 0:
            items = result.get('items', [])
            if items:
                return items[0].get('path')
    except Exception:
        pass
    return None
def cmd_file_history(args):
    if not args.repo or not args.filepath:
        Print.error(t("commands.shared.missing_args"))
        _write_output(f"\n{t('commands.file_history.usage')}")
        return
    headers = get_auth_header()
    repo = args.repo.strip()
    if "/" not in repo or len(repo.split("/")) != 2:
        print()
        _write_output("")
        Print.error(t("commands.shared.invalid_repo_format"))
        print()
        return
    filepath = args.filepath.strip()
    if "/" not in filepath:
        with spinner(t("commands.file_history.searching", filename=filepath, repo=repo), color='96'):
            found_path = _search_file_in_repo(repo, filepath, headers)
        if found_path:
            filepath = found_path
            safe_print("")
            Print.info(t("commands.file_history.found_at", path=filepath))
        else:
            safe_print("")
            Print.error(t("commands.file_history.not_found", filename=filepath))
            safe_print("")
            return
    limit = args.limit
    if limit > 50:
        safe_print("")
        Print.error(t("commands.file_history.limit_exceeded", requested=limit, max=50))
        safe_print("")
        return
    safe_print("")
    with spinner(t("commands.file_history.header", filepath=filepath, repo=repo), color='96'):
        commits_url = f"https://api.github.com/repos/{repo}/commits?path={filepath}&per_page={limit}"
        commits_raw, _ = api_get(commits_url, headers)
    commits = commits_raw if isinstance(commits_raw, list) else []
    commits = commits[:limit]
    from ..helpers import TablePrinter, _truncate
    columns = [
        {'header': t("commands.file_history.table_header.sha"), 'width': 7, 'color': '96'},     
        {'header': t("commands.file_history.table_header.date"), 'width': 10, 'color': '93'},   
        {'header': t("commands.file_history.table_header.author"), 'width': 18, 'color': '92'}, 
        {'header': t("commands.file_history.table_header.message"), 'width': None, 'color': '97'} 
    ]
    printer = TablePrinter(columns)
    printer.print_header()
    for idx, commit in enumerate(commits):
        commit_data = commit['commit']
        style = "2;" if idx % 2 == 1 else ""
        sha_val = commit['sha'][:7]
        try:
            dt = datetime.strptime(commit_data['author']['date'], "%Y-%m-%dT%H:%M:%SZ")
            date_val = dt.strftime("%Y-%m-%d")
        except Exception:
            date_val = commit_data['author']['date'][:10]
        author_val = commit_data['author']['name']
        raw_msg = commit_data['message'].splitlines()[0]
        printer.print_row([sha_val, date_val, author_val, raw_msg], style_prefix=style)
    safe_print("")
    footer_msg = t("commands.file_history.footer", count=len(commits))
    safe_print(Print.colorize(footer_msg, '92'))
    safe_print("")
    _write_output(f"\n{footer_msg}")
